@@ -69,8 +69,9 @@ void Converter::start(const Profile &profile)
         Job job;
         job.disc = Project::instance()->disc(d);
 
-        for (int t = 0; t < job.disc->count(); ++t)
-            job.tracks << job.disc->track(t);
+        for (Track *t : job.disc->tracks()) {
+            job.tracks << t;
+        }
 
         jobs << job;
     }
@@ -86,14 +87,18 @@ void Converter::start(const Converter::Jobs &jobs, const Profile &profile)
     qCDebug(LOG) << "Start converter:" << jobs.length() << "\n"
                  << profile;
     qCDebug(LOG) << "Temp dir =" << profile.tmpDir();
+    qCDebug(LOG) << "Out dir =" << profile.outFileDir();
+    qCDebug(LOG) << "Out profile = " << profile.outFilePattern();
 
     if (jobs.isEmpty()) {
-        emit finished();
+        qCWarning(LOG) << "jobs is empty";
+        emit finished(false);
         return;
     }
 
     if (!validate(jobs, profile)) {
-        emit finished();
+        qCWarning(LOG) << "validation failed";
+        emit finished(false);
         return;
     }
 
@@ -120,7 +125,8 @@ void Converter::start(const Converter::Jobs &jobs, const Profile &profile)
         emit error(err.what());
         qDeleteAll(mDiskPiplines);
         mDiskPiplines.clear();
-        emit finished();
+        qCWarning(LOG) << "starting failed";
+        emit finished(false);
     }
 
     mTotalProgressCounter.init(*this);
@@ -190,13 +196,31 @@ void Converter::startThread()
         }
     }
 
+    bool success = true;
     foreach (DiscPipeline *pipe, mDiskPiplines) {
         if (pipe->isRunning()) {
             return;
         }
+
+        success = success && pipe->isSuccess();
     }
 
-    emit finished();
+    foreach (DiscPipeline *pipe, mDiskPiplines) {
+        qCDebug(LOG) << "Pipeline status:" << pipe->isSuccess();
+
+        for (auto it = pipe->trackStates().begin(); it != pipe->trackStates().end(); ++it) {
+            qCDebug(LOG) << "  track" << it.key() << "status:" << trackStateToString(it.value());
+        }
+    }
+
+    if (success) {
+        qCInfo(LOG) << "conversion successfully finished";
+    }
+    else {
+        qCWarning(LOG) << "conversion failed";
+    }
+
+    emit finished(success);
 }
 
 /************************************************
@@ -220,10 +244,10 @@ bool Converter::validate(const Jobs &jobs, const Profile &profile)
 
     QString s;
     foreach (QString e, errors) {
-        s += QString("<li style='margin-top: 4px;'> %1</li>").arg(e);
+        s += QStringLiteral("<li style='margin-top: 4px;'> %1</li>").arg(e);
     }
 
-    Messages::error(QString("<html>%1<ul>%2</ul></html>")
+    Messages::error(QStringLiteral("<html>%1<ul>%2</ul></html>")
                             .arg(tr("Conversion is not possible:"))
                             .arg(s));
 
