@@ -135,6 +135,23 @@ QProcess *Encoder::createDemph(const QString &outFile)
     return res;
 }
 
+/**************************************
+
+ **************************************/
+void Encoder::stopProcesses(QList<QProcess *> procs)
+{
+    for (QProcess *p : procs) {
+        try {
+            p->closeWriteChannel();
+            p->terminate();
+            p->waitForFinished(-1);
+            delete p;
+        }
+        catch (...) {
+        }
+    }
+}
+
 /************************************************
 
  ************************************************/
@@ -203,7 +220,7 @@ void Encoder::run()
 
         for (QProcess *p : procs) {
             if (p->exitCode() != 0) {
-                throw(QString::fromLocal8Bit(p->readAllStandardError()));
+                throw FlaconError(QString::fromLocal8Bit(p->readAllStandardError()));
             }
         }
 
@@ -212,9 +229,17 @@ void Encoder::run()
 
         emit trackReady(track(), outFile(), mTrackGain.result());
     }
-    catch (const FlaconError &err) {
+    catch (const Abort &) {
+        stopProcesses(procs);
         deleteFile(mInputFile);
-        QString msg = tr("Track %1. Encoder error:", "Track error message, %1 is a track number").arg(track().trackNum()) + "<pre>" + err.what() + "</pre>";
+
+        qCDebug(LOG) << "Encoder job was aborted on track" << track().trackNumTag();
+    }
+    catch (const FlaconError &err) {
+        stopProcesses(procs);
+        deleteFile(mInputFile);
+
+        QString msg = tr("Track %1. Encoder error:", "Track error message, %1 is a track number").arg(track().trackNumTag()) + "<pre>" + err.what() + "</pre>";
         emit    error(track(), msg);
     }
 }
@@ -224,7 +249,7 @@ void Encoder::run()
  ************************************************/
 void Encoder::writeMetadata() const
 {
-    MetadataWriter *writer = mProfile.outFormat()->createMetadataWriter(outFile());
+    MetadataWriter *writer = mProfile.outFormat()->createMetadataWriter(mProfile, outFile());
     if (!writer) {
         return;
     }
@@ -289,6 +314,7 @@ void Encoder::readInputFile(QProcess *process)
     QByteArray buf;
 
     while (!file.atEnd()) {
+        Abort::check();
         buf = file.read(bufSize);
         process->write(buf);
         if (mReplayGainEnabled) {
@@ -323,7 +349,7 @@ QStringList Encoder::resamplerArgs(int bitsPerSample, int sampleRate, const QStr
 
     args << "-"; // Read from STDIN
     if (bitsPerSample) {
-        args << "-b" << QString("%1").arg(bitsPerSample);
+        args << "-b" << QStringLiteral("%1").arg(bitsPerSample);
     }
 
     args << "--type"
@@ -333,7 +359,7 @@ QStringList Encoder::resamplerArgs(int bitsPerSample, int sampleRate, const QStr
     if (sampleRate) {
         args << "rate";
         args << "-v"; // very high quality
-        args << QString("%1").arg(sampleRate);
+        args << QStringLiteral("%1").arg(sampleRate);
     }
 
     return args;
