@@ -65,19 +65,19 @@ Splitter::Job::Job(const Disc *disk, Conv::ConvTrack track, bool addPregap, bool
     disk(disk),
     track(track)
 {
-    const Track cur  = *(disk->track(track.index()));
-    const Track next = (cur.index() + 1 < disk->count()) ? *(disk->track(cur.index() + 1)) : Track();
+    const Track cur  = *(disk->tracks().at(track.index()));
+    const Track next = (cur.index() + 1 < disk->tracks().count()) ? *(disk->tracks().at(cur.index() + 1)) : Track();
 
     if (addPregap) {
-        chunks << getPart(cur.cueIndex(0), cur.cueIndex(1));
+        chunks << getPart(cur.cueIndex00(), cur.cueIndex01());
     }
 
     if (addTrack) {
-        chunks << getPart(cur.cueIndex(1), next.cueIndex(0));
+        chunks << getPart(cur.cueIndex01(), next.cueIndex00());
     }
 
     if (addPostgap) {
-        chunks << getPart(next.cueIndex(0), next.cueIndex(1));
+        chunks << getPart(next.cueIndex00(), next.cueIndex01());
     }
 
     merge();
@@ -138,22 +138,28 @@ InputAudioFile Splitter::Job::getInputAudioFile(const QByteArray &fileTag) const
     QByteArray prevTag;
 
     int n = 0;
-    for (int i = 0; i < disk->count(); ++i) {
-        const Track *t = disk->track(i);
+    for (const Track *t : disk->tracks()) {
 
-        for (int j = 0; j <= 1; ++j) {
-            if (t->cueIndex(j).file() == fileTag) {
-                return disk->audioFiles().at(n);
-            }
+        if (t->cueIndex00().file() == fileTag) {
+            return disk->audioFiles().at(n);
+        }
 
-            if (t->cueIndex(j).file() != prevTag) {
-                n++;
-                prevTag = t->cueIndex(j).file();
-            }
+        if (t->cueIndex00().file() != prevTag) {
+            n++;
+            prevTag = t->cueIndex00().file();
+        }
+
+        if (t->cueIndex01().file() == fileTag) {
+            return disk->audioFiles().at(n);
+        }
+
+        if (t->cueIndex01().file() != prevTag) {
+            n++;
+            prevTag = t->cueIndex01().file();
         }
     }
 
-    throw FlaconError(QString("Incorrect file tag %1").arg(fileTag.data()));
+    throw FlaconError(QStringLiteral("Incorrect file tag %1").arg(fileTag.data()));
 }
 
 /************************************************
@@ -197,7 +203,7 @@ void Splitter::run()
 
         bool addPregap = (track.index() == 0 && mPregapType == PreGapType::AddToFirstTrack);
         Job  job(mDisc, track, addPregap, true, true);
-        job.outFileName = QString("%1/track-%2_%3.wav").arg(mOutDir, uid).arg(track.trackNum(), 2, 10, QLatin1Char('0'));
+        job.outFileName = QString("%1/track-%2_%3.wav").arg(mOutDir, uid).arg(track.trackNumTag(), 2, 10, QLatin1Char('0'));
         jobs << job;
     }
 
@@ -267,12 +273,17 @@ void Splitter::run()
             qCDebug(LOG) << "Splitter trackReady:" << job.track << job.outFileName;
             emit trackReady(job.track, job.outFileName);
         }
+        catch (const Abort &) {
+            qCDebug(LOG) << "Splitter job was aborted on track" << job.track.trackNumTag();
+            deleteFile(job.outFileName);
+            return;
+        }
         catch (FlaconError &err) {
             if (job.isPregap) {
                 qCWarning(LOG) << "Splitter error for pregap track : " << err.what();
             }
             else {
-                qCWarning(LOG) << "Splitter error for track " << job.track.trackNum() << ": " << err.what();
+                qCWarning(LOG) << "Splitter error for track " << job.track.trackNumTag() << ": " << err.what();
             }
 
             emit error(job.track, err.what());
@@ -300,7 +311,7 @@ void Splitter::processTrack(const Job &job)
 
     QFile outFile(job.outFileName);
     if (!outFile.open(QFile::WriteOnly)) {
-        throw outFile.errorString();
+        throw FlaconError(outFile.errorString());
     }
 
     uint32_t bytes = 0;
@@ -318,7 +329,7 @@ void Splitter::processTrack(const Job &job)
     for (const Job::Chunk &chunk : job.chunks) {
         try {
             progress.chunkSize = chunk.decoder->bytesCount(chunk.start, chunk.end);
-
+            Abort::check();
             // Extract chunk .............................
             QObject keeper;
             connect(chunk.decoder, &Decoder::progress, &keeper, [this, job, progress](int percents) {
@@ -330,7 +341,7 @@ void Splitter::processTrack(const Job &job)
             qCDebug(LOG) << "extract: " << chunk.file.filePath() << " [" << chunk.start.toString() << ":" << chunk.end.toString() << "] OUT:" << job.outFileName;
             chunk.decoder->extract(chunk.start, chunk.end, &outFile, false);
         }
-        catch (FlaconError &err) {
+        catch (const FlaconError &err) {
             throw FlaconError(tr("I can't read <b>%1</b>:<br>%2", "Splitter error. %1 is a file name, %2 is a system error text.").arg(chunk.file.fileName(), err.what()));
         }
     }
