@@ -26,7 +26,6 @@
 #include "cue.h"
 #include "cuedata.h"
 #include "types.h"
-#include "tags.h"
 #include <QFile>
 #include <QFileInfo>
 #include <QObject>
@@ -37,15 +36,19 @@
 #include "formats_in/informat.h"
 #include <QBuffer>
 
-/************************************************
- *
- ************************************************/
-static void splitTitle(TrackTags *track, char separator)
+/**************************************
+ * Tags::Track
+ **************************************/
+Tags::Track Cue::Track::decode(const TextCodec &textCodec) const
 {
-    QByteArray b   = track->tagData(TagId::Title);
-    int        pos = b.indexOf(separator);
-    track->setTag(TagId::Artist, b.left(pos).trimmed());
-    track->setTag(TagId::Title, b.right(b.length() - pos - 1).trimmed());
+    Tags::Track res;
+    res.setTrackNum(mTrackNumTag);
+
+    for (TrackTags::TagId tagId : mTags.keys()) {
+        res.setTag(tagId, textCodec.decode(mTags.value(tagId)));
+    }
+
+    return res;
 }
 
 /************************************************
@@ -59,8 +62,9 @@ Cue::Cue(const QString &fileName) noexcept(false)
     }
 
     QFileInfo fileInfo(fileName);
-    mFilePath = fileInfo.absoluteFilePath();
-    mTitle    = fileInfo.fileName();
+    mFilePath     = fileInfo.absoluteFilePath();
+    mTagsId.uri   = fileInfo.absoluteFilePath();
+    mTagsId.title = fileInfo.fileName();
 
     read(data);
 }
@@ -70,19 +74,21 @@ Cue::Cue(const QString &fileName) noexcept(false)
  ************************************************/
 void Cue::read(const CueData &data)
 {
+    const CueData::Tags &global = data.globalTags();
 
-    mDiscCount = 1;
-    mDiscNum   = 1;
-
-    QByteArray           albumPerformer = getAlbumPerformer(data);
-    const CueData::Tags &global         = data.globalTags();
+    mTags[AlbumTags::TagId::Album]          = global.value(CueData::TITLE_TAG);
+    mTags[AlbumTags::TagId::Catalog]        = global.value("CATALOG");
+    mTags[AlbumTags::TagId::CdTextfile]     = global.value("CDTEXTFILE");
+    mTags[AlbumTags::TagId::DiscId]         = global.value("DISCID");
+    mDiscNumTag                             = global.value("DISCNUMBER", "1").toInt();
+    mDiscCountTag                           = global.value("TOTALDISCS", "1").toInt();
+    mTags[AlbumTags::TagId::AlbumPerformer] = getAlbumPerformer(data);
 
     QByteArray prevFileTag;
 
     for (const CueData::Tags &t : data.tracks()) {
-        TrackTags track;
-        track.setTag(TagId::File, t.value(CueData::FILE_TAG));
-        track.setTag(TagId::Album, global.value(CueData::TITLE_TAG));
+        Track track;
+        track.mFileTag = t.value(CueData::FILE_TAG);
 
         QByteArray index00     = t.value("INDEX 00");
         QByteArray index00File = t.value("INDEX 00 FILE");
@@ -107,28 +113,20 @@ void Cue::read(const CueData &data)
         }
         prevFileTag = index01File;
 
-        track.setCueIndex(0, CueIndex(index00, index00File));
-        track.setCueIndex(1, CueIndex(index01, index01File));
+        track.mCueIndex00 = CueIndex(index00, index00File);
+        track.mCueIndex01 = CueIndex(index01, index01File);
 
-        track.setTag(TagId::Catalog, global.value("CATALOG"));
-        track.setTag(TagId::CDTextfile, global.value("CDTEXTFILE"));
-        track.setTag(TagId::Comment, global.value("COMMENT"));
+        track.mTags[TrackTags::TagId::Flags] = t.value(CueData::FLAGS_TAG);
+        track.mTags[TrackTags::TagId::Isrc]  = t.value("ISRC");
+        track.mTags[TrackTags::TagId::Title] = t.value("TITLE");
 
-        track.setTag(TagId::Flags, t.value(CueData::FLAGS_TAG));
-        track.setTag(TagId::Genre, global.value("GENRE"));
-        track.setTag(TagId::ISRC, t.value("ISRC"));
-        track.setTag(TagId::Title, t.value("TITLE"));
-        track.setTag(TagId::Artist, t.value("PERFORMER", global.value("PERFORMER")));
-        track.setTag(TagId::SongWriter, t.value("SONGWRITER", global.value("SONGWRITER")));
-        track.setTag(TagId::DiscId, global.value("DISCID"));
+        track.mTags[TrackTags::TagId::Comment]    = t.value("COMMENT", global.value("COMMENT"));
+        track.mTags[TrackTags::TagId::Genre]      = t.value("GENRE", global.value("GENRE"));
+        track.mTags[TrackTags::TagId::Performer]  = t.value("PERFORMER", global.value("PERFORMER"));
+        track.mTags[TrackTags::TagId::SongWriter] = t.value("SONGWRITER", global.value("SONGWRITER"));
+        track.mTags[TrackTags::TagId::Date]       = t.value("DATE", global.value("DATE"));
 
-        track.setTag(TagId::Date, t.value("DATE", global.value("DATE")));
-        track.setTag(TagId::AlbumArtist, albumPerformer);
-
-        track.setTrackNum(TrackNum(t.value(CueData::TRACK_TAG).toUInt()));
-        track.setTrackCount(TrackNum(data.tracks().count()));
-        track.setTag(TagId::DiscNum, global.value("DISCNUMBER", "1"));
-        track.setTag(TagId::DiscCount, global.value("TOTALDISCS", "1"));
+        track.mTrackNumTag = TrackNum(t.value(CueData::TRACK_TAG).toUInt());
 
         mTracks.append(track);
     }
@@ -139,16 +137,21 @@ void Cue::read(const CueData &data)
 
     {
         mFileTags.clear();
-        QByteArray prev;
-        for (const TrackTags &track : mTracks) {
-            if (track.tagValue(TagId::File).value() != prev) {
-                mFileTags << QString::fromUtf8(track.tagData(TagId::File));
-                prev = track.tagValue(TagId::File).value();
+        mMutiplyAudio = false;
+
+        QString prev;
+        for (const Track &track : qAsConst(mTracks)) {
+            QString s = QString::fromUtf8(track.fileTag());
+
+            if (s != prev) {
+                mMutiplyAudio = !mFileTags.isEmpty();
+                mFileTags << s;
+                prev = s;
             }
         }
     }
 
-    setCodecName(data);
+    mBom = data.bomCodec();
     validate();
 }
 
@@ -158,8 +161,8 @@ void Cue::read(const CueData &data)
 void Cue::validate()
 {
     bool hasFileTag = false;
-    for (const TrackTags &t : mTracks) {
-        hasFileTag = hasFileTag || !t.tagData(TagId::File).isEmpty();
+    for (const Track &t : qAsConst(mTracks)) {
+        hasFileTag = hasFileTag || !t.fileTag().isEmpty();
     }
 
     if (!hasFileTag) {
@@ -170,18 +173,51 @@ void Cue::validate()
 /************************************************
  *
  ************************************************/
-bool Cue::isMutiplyAudio() const
+void Cue::splitTitle(Track *track, char separator)
 {
-    if (isEmpty())
-        return false;
+    QByteArray b                              = track->titleTag();
+    int        pos                            = b.indexOf(separator);
+    track->mTags[TrackTags::TagId::Performer] = b.left(pos).trimmed();
+    track->mTags[TrackTags::TagId::Title]     = b.right(b.length() - pos - 1).trimmed();
+}
 
-    QByteArray file = mTracks.last().tagData(TagId::File);
-    for (const TrackTags &track : mTracks) {
-        if (track.tagData(TagId::File) != file)
-            return true;
+/************************************************
+ *
+ ************************************************/
+TextCodec Cue::detectTextCodec() const
+{
+    UcharDet uchardet;
+
+    uchardet << albumTag();
+
+    for (const Track &track : mTracks) {
+        uchardet << track.performerTag();
+        uchardet << track.titleTag();
     }
 
-    return false;
+    return uchardet.detect();
+}
+
+/**************************************
+ *
+ **************************************/
+Tags Cue::decode(const TextCodec &textCodec) const
+{
+    Tags res;
+    // clang-format off
+    if (mDiscCountTag)  res.setDiscCount(mDiscCountTag);
+    if (mDiscNumTag)    res.setDiscNum(mDiscNumTag);
+    // clang-format on
+
+    for (AlbumTags::TagId tagId : mTags.keys()) {
+        res.setTag(tagId, textCodec.decode(mTags.value(tagId)));
+    }
+
+    for (const Track &t : mTracks) {
+        res.tracks().append(t.decode(textCodec));
+    }
+
+    return res;
 }
 
 /************************************************
@@ -224,7 +260,7 @@ void Cue::splitTitleTag(const CueData &data)
     }
 
     // clang-format off
-    for (TrackTags &track : mTracks) {
+    for (Track &track : mTracks) {
         if (splitByBackSlash)  splitTitle(&track, '\\');
         else if (splitBySlash) splitTitle(&track, '/');
         else if (splitByDash)  splitTitle(&track, '-');
@@ -232,31 +268,6 @@ void Cue::splitTitleTag(const CueData &data)
     // clang-format on
 }
 
-/************************************************
- * Auto detect codepage
- ************************************************/
-void Cue::setCodecName(const CueData &data)
-{
-    TextCodec codec;
-
-    TextCodec::BomCodec bom = data.bomCodec();
-    if (bom != TextCodec::BomCodec::Unknown) {
-        codec = TextCodec::codecForMib(int(bom));
-    }
-    else {
-        UcharDet charDet;
-        foreach (const TrackTags &track, mTracks) {
-            charDet << track;
-        }
-
-        codec = TextCodec::codecForName(charDet.textCodecName());
-    }
-
-    for (TrackTags &track : mTracks) {
-        track.setCodec(codec);
-    }
-}
-
 /************************************************
  *
  ************************************************/
@@ -264,8 +275,9 @@ EmbeddedCue::EmbeddedCue(const InputAudioFile &audioFile) noexcept(false) :
     Cue()
 {
     QFileInfo fileInfo(audioFile.filePath());
-    mFilePath = EMBEDED_PREFIX + fileInfo.absoluteFilePath();
-    mTitle    = QObject::tr("Embedded on %1", "The title for the CUE embedded in the audio file. %1 - is an audio-file name.").arg(fileInfo.fileName());
+    mFilePath     = EMBEDED_PREFIX + fileInfo.absoluteFilePath();
+    mTagsId.uri   = mFilePath;
+    mTagsId.title = QObject::tr("Embedded on %1", "The title for the CUE embedded in the audio file. %1 - is an audio-file name.").arg(fileInfo.fileName());
 
     if (!audioFile.isValid() || !audioFile.format()) {
         return;
@@ -287,9 +299,9 @@ EmbeddedCue::EmbeddedCue(const InputAudioFile &audioFile) noexcept(false) :
     read(data);
 }
 
-/************************************************
+/**************************************
  *
- ************************************************/
+ **************************************/
 CueError::~CueError()
 {
 }
