@@ -159,7 +159,7 @@ OutPatternButton::OutPatternButton(QWidget *parent) :
  ************************************************/
 void OutPatternButton::addPattern(const QString &pattern, const QString &title)
 {
-    QAction *act = new QAction(title + QString(" (%1)").arg(pattern), this);
+    QAction *act = new QAction(title + QStringLiteral(" (%1)").arg(pattern), this);
     act->setData(pattern);
     connect(act, &QAction::triggered,
             this, &OutPatternButton::patternTriggered);
@@ -186,7 +186,7 @@ void OutPatternButton::addStandardPatterns()
 {
     addPattern("%n", tr("Insert \"Track number\""));
     addPattern("%N", tr("Insert \"Total number of tracks\""));
-    addPattern("%a", tr("Insert \"Album Performer\""));
+    addPattern("%a", tr("Insert \"Album performer\""));
     addPattern("%A", tr("Insert \"Album title\""));
     addPattern("%t", tr("Insert \"Track title\""));
     addPattern("%y", tr("Insert \"Year\""));
@@ -286,7 +286,7 @@ void OutDirButton::fillMenu()
         if (!dir.isEmpty() && dir != QDir::homePath()) {
             QAction *act = new QAction(menu);
             act->setText(tr("Standard music location", "Menu item for output direcory button"));
-            connect(act, &QAction::triggered, [this, dir]() { setDirectory(dir); });
+            connect(act, &QAction::triggered, this, [this, dir]() { setDirectory(dir); });
             menu->addAction(act);
         }
     }
@@ -296,7 +296,7 @@ void OutDirButton::fillMenu()
         if (!dir.isEmpty()) {
             QAction *act = new QAction(menu);
             act->setText(tr("Desktop", "Menu item for output direcory button"));
-            connect(act, &QAction::triggered, [this, dir]() { setDirectory(dir); });
+            connect(act, &QAction::triggered, this, [this, dir]() { setDirectory(dir); });
             menu->addAction(act);
         }
     }
@@ -304,7 +304,7 @@ void OutDirButton::fillMenu()
     {
         QAction *act = new QAction(menu);
         act->setText(tr("Same directory as CUE file", "Menu item for output direcory button"));
-        connect(act, &QAction::triggered, [this]() { setDirectory(""); });
+        connect(act, &QAction::triggered, this, [this]() { setDirectory(""); });
         menu->addAction(act);
     }
 }
@@ -431,7 +431,7 @@ MultiValuesSpinBox::MultiValuesSpinBox(QWidget *parent) :
 /************************************************
  *
  ************************************************/
-static QString getTagEditText(const QSet<QString> &values)
+static QString getMultiValuesText(const QSet<QString> &values)
 {
     switch (values.count()) {
         case 0:
@@ -443,21 +443,6 @@ static QString getTagEditText(const QSet<QString> &values)
     }
 }
 
-/************************************************
- *
- ************************************************/
-static QString getTagEditPlaceHolder(const QSet<QString> &values)
-{
-    switch (values.count()) {
-        case 0:
-            return "";
-        case 1:
-            return "";
-        default:
-            return QObject::tr("Multiple values");
-    }
-}
-
 /************************************************
 
  ************************************************/
@@ -552,31 +537,182 @@ MultiValuesLineEdit::MultiValuesLineEdit(QWidget *parent) :
     completer()->setModel(mCompleterModel);
     completer()->setCaseSensitivity(Qt::CaseInsensitive);
     completer()->setCompletionMode(QCompleter::PopupCompletion);
+
+    connect(this, &QLineEdit::textEdited, this, &MultiValuesLineEdit::onTextChanged);
 }
 
-/************************************************
-
- ************************************************/
 void MultiValuesLineEdit::setMultiValue(QSet<QString> value)
 {
-    QString str = value.count() == 1 ? *(value.constBegin()) : "";
+    mValues = value;
+    refresh();
+}
 
+QString MultiValuesLineEdit::getPlaceholderText(int)
+{
+    return tr("Multiple values", "MultiValues edit placeholder");
+}
+
+void MultiValuesLineEdit::onTextChanged()
+{
+    QSet<QString> val;
+    val.insert(text());
+    setMultiValue(val);
+}
+
+void MultiValuesLineEdit::refresh()
+{
+    setPlaceholderText(mValues.count() > 1 ? getPlaceholderText(mValues.count()) : "");
+
+    QString str = getMultiValuesText(mValues);
     if (text() != str) {
         QLineEdit::setText(str);
     }
 
-    setPlaceholderText(value.count() > 1 ? tr("Multiple values") : "");
-    mCompleterModel->setStringList(value.values());
+    mCompleterModel->setStringList(mValues.values());
+}
+
+/**************************************
+ * TrackTagLineEdit
+ **************************************/
+TrackTagLineEdit::TrackTagLineEdit(QWidget *parent) :
+    MultiValuesLineEdit(parent)
+{
+}
+
+void TrackTagLineEdit::loadFromTracks(const TrackPtrList &tracks)
+{
+    QSet<QString> values;
+    for (const Track *track : tracks) {
+        values << track->tag(mTagId);
+    }
+
+    setMultiValue(values);
+}
+
+void TrackTagLineEdit::writeToTracks(const TrackPtrList &tracks)
+{
+    if (!isModified()) {
+        return;
+    }
+
+    for (Track *track : tracks) {
+        track->setTag(mTagId, text());
+    }
+}
+
+QString TrackTagLineEdit::getPlaceholderText(int count)
+{
+    return tr("Different across %n songs", "Placeholder for tags edit, %n is count of different values", count);
+}
+
+/**************************************
+ * AlbumTagLineEdit
+ **************************************/
+AlbumTagLineEdit::AlbumTagLineEdit(QWidget *parent) :
+    MultiValuesLineEdit(parent)
+{
+}
+
+void AlbumTagLineEdit::loadFromDisks(const DiskList &disks)
+{
+    QSet<QString> values;
+    for (const Disk *disk : disks) {
+        values << disk->tag(mTagId);
+    }
+
+    setMultiValue(values);
+}
+
+void AlbumTagLineEdit::writeToDisks(const DiskList &disks)
+{
+    if (!isModified()) {
+        return;
+    }
+
+    for (Disk *disk : disks) {
+        disk->setTag(mTagId, text());
+    }
+}
+
+QString AlbumTagLineEdit::getPlaceholderText(int count)
+{
+    return tr("Different across %n disks", "Placeholder for tags edit, %n is count of different values", count);
 }
 
 /************************************************
- *
+ * MultiValuesTextEdit
  ************************************************/
-
-TagLineEdit::TagLineEdit(QWidget *parent) :
-    MultiValuesLineEdit(parent),
-    mTagId(TagId())
+MultiValuesTextEdit::MultiValuesTextEdit(QWidget *parent) :
+    QPlainTextEdit(parent)
 {
+    connect(this, &QPlainTextEdit::textChanged, this, &MultiValuesTextEdit::onTextChanged);
+}
+
+bool MultiValuesTextEdit::isModified() const
+{
+    return this->document()->isModified();
+}
+
+void MultiValuesTextEdit::setMultiValue(QSet<QString> values)
+{
+    mValues = values;
+    refresh();
+}
+
+void MultiValuesTextEdit::onTextChanged()
+{
+    QSet<QString> val;
+    val.insert(text());
+    setMultiValue(val);
+}
+
+void MultiValuesTextEdit::refresh()
+{
+    setPlaceholderText(mValues.count() > 1 ? getPlaceholderText(mValues.count()) : "");
+
+    QString str = getMultiValuesText(mValues);
+    if (str != toPlainText()) {
+        setPlainText(str);
+    }
+}
+
+QString MultiValuesTextEdit::getPlaceholderText(int)
+{
+    return tr("Multiple values", "MultiValues edit placeholder");
+}
+
+/**************************************
+ * TrackTagTextEdit
+ **************************************/
+TrackTagTextEdit::TrackTagTextEdit(QWidget *parent) :
+    MultiValuesTextEdit(parent)
+{
+}
+
+void TrackTagTextEdit::loadFromTracks(const TrackPtrList &tracks)
+{
+    QSet<QString> values;
+    for (const Track *track : tracks) {
+        values << track->tag(mTagId);
+    }
+
+    setMultiValue(values);
+}
+
+void TrackTagTextEdit::writeToTracks(const TrackPtrList &tracks)
+{
+    if (!isModified()) {
+        return;
+    }
+
+    for (Track *track : tracks) {
+        track->setTag(mTagId, text());
+    }
+}
+
+QString TrackTagTextEdit::getPlaceholderText(int count)
+{
+    return tr("Different across %n songs", "Placeholder for tags edit, %n is count of different values", count);
 }
 
 /************************************************
@@ -675,7 +811,7 @@ void ProgramEdit::openDialog()
     QString flt = tr("%1 program",
                      "This is part of filter for 'select program' dialog. %1 is a name of required program. Example: 'flac program (flac)'")
                           .arg(mProgram->name())
-            + QString(" (%1);;").arg(mProgram->name()) + tr("All files", "This is part of filter for 'select program' dialog. 'All files (*)'") + " (*)";
+            + QStringLiteral(" (%1);;").arg(mProgram->name()) + tr("All files", "This is part of filter for 'select program' dialog. 'All files (*)'") + " (*)";
 
     QString fileName = !text().isEmpty() ? text() : "/usr/bin/" + mProgram->name();
 
@@ -806,45 +942,10 @@ OutDirComboBox::OutDirComboBox(QWidget *parent) :
 /************************************************
  *
  ************************************************/
-TagSpinBox::TagSpinBox(QWidget *parent) :
-    MultiValuesSpinBox(parent),
-    mTagId(TagId())
-{
-}
 
 /************************************************
  *
  ************************************************/
-MultiValuesTextEdit::MultiValuesTextEdit(QWidget *parent) :
-    QPlainTextEdit(parent)
-{
-}
-
-/************************************************
- *
- ************************************************/
-bool MultiValuesTextEdit::isModified() const
-{
-    return this->document()->isModified();
-}
-
-/************************************************
- *
- ************************************************/
-void MultiValuesTextEdit::setMultiValue(QSet<QString> value)
-{
-    setPlainText(getTagEditText(value));
-    setPlaceholderText(getTagEditPlaceHolder(value));
-}
-
-/************************************************
- *
- ************************************************/
-TagTextEdit::TagTextEdit(QWidget *parent) :
-    MultiValuesTextEdit(parent),
-    mTagId(TagId())
-{
-}
 
 /************************************************
  *
@@ -871,7 +972,7 @@ void ErrorBox::setMessages(const QStringList &messages)
     QString txt;
     foreach (QString s, mMessgaes) {
         s.replace("\n", "<p>");
-        txt += QString("<li>%1</li>").arg(s);
+        txt += QStringLiteral("<li>%1</li>").arg(s);
     }
 
     setText("<ul>" + txt + "</ul>");
