@@ -36,6 +36,7 @@
 #include <QMessageBox>
 #include <QLabel>
 #include "types.h"
+#include "disc.h"
 
 class QStringListModel;
 class QToolBar;
@@ -171,11 +172,62 @@ class MultiValuesLineEdit : public QLineEdit
 public:
     explicit MultiValuesLineEdit(QWidget *parent = nullptr);
 
-public slots:
-    void setMultiValue(QSet<QString> value);
+    QSet<QString> multiValues() { return mValues; }
+    void          setMultiValue(QSet<QString> value);
+
+protected:
+    virtual QString getPlaceholderText(int count);
 
 private:
+    QSet<QString>     mValues;
     QStringListModel *mCompleterModel;
+
+    void onTextChanged();
+    void refresh();
+};
+
+/************************************************
+
+ ************************************************/
+class TrackTagLineEdit : public MultiValuesLineEdit
+{
+    Q_OBJECT
+public:
+    explicit TrackTagLineEdit(QWidget *parent = nullptr);
+
+    TrackTags::TagId tagId() const { return mTagId; }
+    void             setTagId(TrackTags::TagId value) { mTagId = value; }
+
+    void loadFromTracks(const TrackPtrList &tracks);
+    void writeToTracks(const TrackPtrList &tracks);
+
+protected:
+    QString getPlaceholderText(int count) override;
+
+private:
+    TrackTags::TagId mTagId = TrackTags::TagId(-1);
+};
+
+/************************************************
+
+ ************************************************/
+class AlbumTagLineEdit : public MultiValuesLineEdit
+{
+    Q_OBJECT
+public:
+    AlbumTagLineEdit(QWidget *parent = nullptr);
+
+    AlbumTags::TagId tagId() const { return mTagId; }
+    void             setTagId(AlbumTags::TagId value) { mTagId = value; }
+
+    void loadFromDisks(const DiskList &disks);
+    void writeToDisks(const DiskList &disks);
+
+protected:
+    QString getPlaceholderText(int count) override;
+
+private:
+    AlbumTags::TagId mTagId = AlbumTags::TagId(-1);
 };
 
 /************************************************
@@ -191,55 +243,38 @@ public:
     QString text() const { return this->toPlainText(); }
 
 public slots:
-    void setMultiValue(QSet<QString> value);
-};
+    void setMultiValue(QSet<QString> values);
 
-/************************************************
- *
- ************************************************/
-class TagTextEdit : public MultiValuesTextEdit
-{
-    Q_OBJECT
-public:
-    explicit TagTextEdit(QWidget *parent = nullptr);
-
-    TagId tagId() const { return mTagId; }
-    void  setTagId(TagId tagId) { mTagId = tagId; }
+protected:
+    virtual QString getPlaceholderText(int count);
 
 private:
-    TagId mTagId;
+    QSet<QString> mValues;
+
+    void onTextChanged();
+    void refresh();
 };
 
 /************************************************
 
  ************************************************/
-class TagLineEdit : public MultiValuesLineEdit
+class TrackTagTextEdit : public MultiValuesTextEdit
 {
     Q_OBJECT
 public:
-    explicit TagLineEdit(QWidget *parent = nullptr);
+    explicit TrackTagTextEdit(QWidget *parent = nullptr);
 
-    TagId tagId() const { return mTagId; }
-    void  setTagId(TagId tagId) { mTagId = tagId; }
+    TrackTags::TagId tagId() const { return mTagId; }
+    void             setTagId(TrackTags::TagId value) { mTagId = value; }
+
+    void loadFromTracks(const TrackPtrList &tracks);
+    void writeToTracks(const TrackPtrList &tracks);
+
+protected:
+    QString getPlaceholderText(int count) override;
 
 private:
-    TagId mTagId;
-};
-
-/************************************************
- *
- ************************************************/
-class TagSpinBox : public MultiValuesSpinBox
-{
-    Q_OBJECT
-public:
-    explicit TagSpinBox(QWidget *parent = nullptr);
-
-    TagId tagId() const { return mTagId; }
-    void  setTagId(TagId tagId) { mTagId = tagId; }
-
-private:
-    TagId mTagId;
+    TrackTags::TagId mTagId = TrackTags::TagId(-1);
 };
 
 /************************************************
