@@ -23,13 +23,12 @@
  *
  * END_COMMON_COPYRIGHT_HEADER */
 
-
 #ifndef ICON_H
 #define ICON_H
 
 #include <QIcon>
 
-class Icon: public QIcon
+class Icon : public QIcon
 {
 public:
     Icon();
@@ -40,4 +39,11 @@ public:
     static void setDarkMode(bool dark);
 };
 
+class Pixmap : public QPixmap
+{
+public:
+    Pixmap(const QString &name, QSize size);
+    Pixmap(const QString &name, int width, int height);
+};
+
 #endif // ICON_H
