@@ -41,6 +41,7 @@
 #include "gui/icon.h"
 #include "application.h"
 #include "gui/messagebox/messagebox.h"
+#include "gui/logview/logview.h"
 
 #include <QFileDialog>
 #include <QDir>
@@ -102,29 +103,17 @@ MainWindow::MainWindow(QWidget *parent) :
     trackView->setRootIsDecorated(false);
     trackView->setItemsExpandable(false);
     trackView->hideColumn((int)TrackView::ColumnComment);
+    trackView->hideColumn((int)TrackView::ColumnSongWriter);
+    trackView->hideColumn((int)TrackView::ColumnDate);
+    trackView->hideColumn((int)TrackView::ColumnGenre);
     trackView->setAlternatingRowColors(false);
 
     // Tag edits ...............................................
-    tagGenreEdit->setTagId(TagId::Genre);
-    connect(tagGenreEdit, &TagLineEdit::textEdited, this, &MainWindow::setTrackTag);
-
-    tagYearEdit->setTagId(TagId::Date);
-    connect(tagYearEdit, &TagLineEdit::textEdited, this, &MainWindow::setTrackTag);
-
-    tagArtistEdit->setTagId(TagId::Artist);
-    connect(tagArtistEdit, &TagLineEdit::textEdited, this, &MainWindow::setTrackTag);
-    connect(tagArtistEdit, &TagLineEdit::textEdited, this, &MainWindow::refreshEdits);
-
-    tagDiscPerformerEdit->setTagId(TagId::AlbumArtist);
-    connect(tagDiscPerformerEdit, &TagLineEdit::textEdited, this, &MainWindow::setDiscTag);
-    connect(tagDiscPerformerEdit, &TagLineEdit::textEdited, this, &MainWindow::refreshEdits);
-
-    tagAlbumEdit->setTagId(TagId::Album);
-    connect(tagAlbumEdit, &TagLineEdit::textEdited, this, &MainWindow::setTrackTag);
-
-    tagDiscIdEdit->setTagId(TagId::DiscId);
-    connect(tagDiscIdEdit, &TagLineEdit::textEdited, this, &MainWindow::setTrackTag);
-    connect(tagDiscIdEdit, &TagLineEdit::textEdited, this, &MainWindow::setControlsEnable);
+    connect(tagGenreEdit, &QLineEdit::textEdited, this, &MainWindow::setGenreTag);
+    connect(tagYearEdit, &QLineEdit::textEdited, this, &MainWindow::setDateTag);
+    connect(tagPerformerEdit, &QLineEdit::textEdited, this, &MainWindow::setPerformerTag);
+    connect(tagAlbumPerformerEdit, &QLineEdit::textEdited, this, &MainWindow::setAlbumPerformerTag);
+    connect(tagAlbumEdit, &QLineEdit::textEdited, this, &MainWindow::setAlbumTag);
 
     connect(tagStartNumEdit, &MultiValuesSpinBox::editingFinished, this, &MainWindow::setStartTrackNum);
     connect(tagStartNumEdit, qOverload<int>(&MultiValuesSpinBox::valueChanged),
@@ -161,7 +150,9 @@ MainWindow::MainWindow(QWidget *parent) :
 
     // Signals .................................................
     connect(outDirEdit->lineEdit(), &QLineEdit::textEdited, this, &MainWindow::setOutDir);
+    connect(outDirEdit->lineEdit(), &QLineEdit::editingFinished, this, &MainWindow::setOutDir);
     connect(outPatternEdit->lineEdit(), &QLineEdit::textEdited, this, &MainWindow::setPattern);
+    connect(outPatternEdit->lineEdit(), &QLineEdit::editingFinished, this, &MainWindow::setPattern);
 
     connect(outProfileCombo, qOverload<int>(&QComboBox::currentIndexChanged),
             this, &MainWindow::setOutProfile);
@@ -195,6 +186,10 @@ MainWindow::MainWindow(QWidget *parent) :
                 Icon::setDarkMode(Application::instance()->isDarkVisualMode());
             });
 
+    Validator *validator = &Project::instance()->validator();
+    connect(trackView->model(), &TrackViewModel::dataChanged, validator, &Validator::revalidate);
+    connect(trackView->model(), &TrackViewModel::layoutChanged, validator, &Validator::revalidate);
+
     Icon::setDarkMode(Application::instance()->isDarkVisualMode());
 
     loadSettings();
@@ -449,8 +444,12 @@ void MainWindow::refreshEdits()
 
     // Discs ...............................
     QSet<int>     startNums;
-    QSet<QString> discId;
     QSet<QString> codePage;
+    QSet<QString> genre;
+    QSet<QString> album;
+    QSet<QString> date;
+    QSet<QString> performer;
+    QSet<QString> albumPerformer;
 
     bool hasErrors   = false;
     bool hasWarnings = false;
@@ -462,34 +461,24 @@ void MainWindow::refreshEdits()
     QList<Disc *> discs = trackView->selectedDiscs();
     foreach (Disc *disc, discs) {
         startNums << disc->startTrackNum();
-        discId << disc->discId();
         codePage << disc->codecName();
+        album << disc->albumTag();
+        albumPerformer << disc->albumPerformerTag();
     }
 
-    // Tracks ..............................
-    QSet<QString> genre;
-    QSet<QString> artist;
-    QSet<QString> album;
-    QSet<QString> date;
-    QSet<QString> discPerformer;
-
-    QList<Track *> tracks = trackView->selectedTracks();
-    foreach (Track *track, tracks) {
-        genre << track->genre();
-        artist << track->artist();
-        album << track->album();
-        date << track->date();
-        discPerformer << track->tag(TagId::AlbumArtist);
+    for (const Track *track : trackView->selectedTracks()) {
+        genre << track->genreTag();
+        date << track->dateTag();
+        performer << track->performerTag();
     }
 
     tagGenreEdit->setMultiValue(genre);
     tagYearEdit->setMultiValue(date);
-    tagArtistEdit->setMultiValue(artist);
+    tagPerformerEdit->setMultiValue(performer);
+    tagAlbumPerformerEdit->setMultiValue(albumPerformer);
     tagAlbumEdit->setMultiValue(album);
     tagStartNumEdit->setMultiValue(startNums);
-    tagDiscIdEdit->setMultiValue(discId);
     codepageCombo->setMultiValue(codePage);
-    tagDiscPerformerEdit->setMultiValue(discPerformer);
 
     const Profile *profile = Project::instance()->profile();
     if (outDirEdit->currentText() != profile->outFileDir()) {
@@ -568,66 +557,6 @@ void MainWindow::setCodePage()
     }
 }
 
-/************************************************
-
- ************************************************/
-void MainWindow::setTrackTag()
-{
-    TagLineEdit *edit = qobject_cast<TagLineEdit *>(sender());
-    if (!edit)
-        return;
-
-    QList<Disc *> disks = trackView->selectedDiscs();
-    for (Disk *d : disks) {
-        d->blockSignals(true);
-    }
-    QList<Track *> tracks = trackView->selectedTracks();
-    foreach (Track *track, tracks) {
-        track->setTag(edit->tagId(), edit->text());
-    }
-
-    for (Disk *d : disks) {
-        d->blockSignals(false);
-        emit d->tagChanged();
-    }
-
-    trackView->updateAll();
-}
-
-/************************************************
- *
- ************************************************/
-void MainWindow::setDiscTag()
-{
-    TagLineEdit *edit = qobject_cast<TagLineEdit *>(sender());
-    if (!edit)
-        return;
-
-    QList<Disc *> discs = trackView->selectedDiscs();
-    foreach (Disc *disc, discs) {
-        disc->setDiscTag(edit->tagId(), edit->text());
-    }
-
-    trackView->updateAll();
-}
-
-/************************************************
- *
- ************************************************/
-void MainWindow::setDiscTagInt()
-{
-    TagSpinBox *spinBox = qobject_cast<TagSpinBox *>(sender());
-    if (!spinBox)
-        return;
-
-    QList<Disc *> discs = trackView->selectedDiscs();
-    foreach (Disc *disc, discs) {
-        disc->setDiscTag(spinBox->tagId(), QString::number(spinBox->value()));
-    }
-
-    trackView->updateAll();
-}
-
 /************************************************
  *
  ************************************************/
@@ -638,8 +567,8 @@ void MainWindow::startConvertAll()
         Conv::Converter::Job job;
         job.disc = Project::instance()->disc(d);
 
-        for (int t = 0; t < job.disc->count(); ++t) {
-            job.tracks << job.disc->track(t);
+        for (int t = 0; t < job.disc->tracks().count(); ++t) {
+            job.tracks << job.disc->tracks().at(t);
         }
 
         jobs << job;
@@ -658,8 +587,7 @@ void MainWindow::startConvertSelected()
         Conv::Converter::Job job;
         job.disc = disc;
 
-        for (int t = 0; t < disc->count(); ++t) {
-            Track *track = disc->track(t);
+        for (const Track *track : disc->tracks()) {
             if (trackView->isSelected(*track)) {
                 job.tracks << track;
             }
@@ -697,8 +625,8 @@ void MainWindow::startConvert(const Conv::Converter::Jobs &jobs)
 
     for (int d = 0; d < Project::instance()->count(); ++d) {
         Disc *disc = Project::instance()->disc(d);
-        for (int t = 0; t < disc->count(); ++t)
-            trackView->model()->trackProgressChanged(*disc->track(t), TrackState::NotRunning, 0);
+        for (int t = 0; t < disc->tracks().count(); ++t)
+            trackView->model()->trackProgressChanged(*disc->tracks().at(t), TrackState::NotRunning, 0);
     }
 
     trackView->setColumnWidth(TrackView::ColumnPercent, 200);
@@ -779,7 +707,7 @@ QString MainWindow::getOpenFileFilter(bool includeAudio, bool includeCue)
 
     if (includeAudio) {
         foreach (const InputFormat *format, InputFormat::allFormats()) {
-            allFlt << QString(" *.%1").arg(format->ext());
+            allFlt << QStringLiteral(" *.%1").arg(format->ext());
             flt << fltPattern.arg(format->name(), format->ext());
         }
     }
@@ -787,7 +715,7 @@ QString MainWindow::getOpenFileFilter(bool includeAudio, bool includeCue)
     flt.sort();
 
     if (includeCue) {
-        allFlt << QString("*.cue");
+        allFlt << QStringLiteral("*.cue");
         flt.insert(0, fltPattern.arg("CUE", "cue"));
     }
 
@@ -856,6 +784,7 @@ void MainWindow::setAudioForDisc(Disc *disc, int audioFileNum)
 
     disc->setAudioFile(audio, audioFileNum);
     trackView->update(*disc);
+    Project::instance()->validator().revalidate();
 }
 
 /************************************************
@@ -875,7 +804,7 @@ void MainWindow::downloadDiscInfo(Disc *disc)
         return;
     }
 
-    DataProvider *provider = new DataProvider();
+    DataProvider *provider = new DataProvider(Project::instance()->profile());
     connect(provider, &DataProvider::finished,
             provider, &DataProvider::deleteLater);
 
@@ -883,7 +812,7 @@ void MainWindow::downloadDiscInfo(Disc *disc)
             [disc, this]() { trackView->downloadFinished(*disc); });
 
     connect(provider, &DataProvider::finished, provider,
-            [disc](const QVector<Tracks> data) { disc->addTagSets(data); });
+            [disc](const QVector<InternetTags> data) { disc->addInternetTags(data); });
 
     provider->start(*disc);
     trackView->downloadStarted(*disc);
@@ -979,6 +908,23 @@ void MainWindow::openAboutDialog()
     dlg.exec();
 }
 
+/************************************************
+
+ ************************************************/
+void MainWindow::openLogViewDialog()
+{
+    LogView *dialog = findChild<LogView *>();
+
+    if (!dialog) {
+        dialog = new LogView(this);
+    }
+
+    dialog->show();
+    dialog->raise();
+    dialog->activateWindow();
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+}
+
 /************************************************
 
  ************************************************/
@@ -1006,12 +952,12 @@ void MainWindow::fillAudioMenu(Disc *disc, QMenu &menu)
             QString msg;
             if (l.count() == 1) {
                 msg = tr("Select another audio file for %1 track…", "context menu. Placeholders are track number")
-                              .arg(l.first()->trackNum());
+                              .arg(l.first()->trackNumTag());
             }
             else {
                 msg = tr("Select another audio file for tracks %1 to %2…", "context menu. Placeholders are track numbers")
-                              .arg(l.first()->trackNum())
-                              .arg(l.last()->trackNum());
+                              .arg(l.first()->trackNumTag())
+                              .arg(l.last()->trackNumTag());
             }
 
             act = new QAction(msg, &menu);
@@ -1071,7 +1017,7 @@ void MainWindow::showDiskAudioFileMenu(Disc *disc, const QPoint &pos)
  ************************************************/
 void MainWindow::openEditTagsDialog()
 {
-    TagEditor editor(trackView->selectedTracks(), trackView->selectedDiscs(), this);
+    TagEditor editor(trackView->selectedTracks(), this);
     editor.exec();
     refreshEdits();
     setControlsEnable();
@@ -1158,6 +1104,61 @@ void MainWindow::setStartTrackNum()
     }
 }
 
+/**************************************
+ *
+ **************************************/
+void MainWindow::setGenreTag()
+{
+    for (Track *track : trackView->selectedTracks()) {
+        track->setGenreTag(tagGenreEdit->text());
+    }
+    trackView->updateAll();
+}
+
+/**************************************
+ *
+ **************************************/
+void MainWindow::setDateTag()
+{
+    for (Track *track : trackView->selectedTracks()) {
+        track->setDateTag(tagYearEdit->text());
+    }
+    trackView->updateAll();
+}
+
+/**************************************
+ *
+ **************************************/
+void MainWindow::setPerformerTag()
+{
+    for (Track *track : trackView->selectedTracks()) {
+        track->setPerformerTag(tagPerformerEdit->text());
+    }
+    trackView->updateAll();
+}
+
+/**************************************
+ *
+ **************************************/
+void MainWindow::setAlbumPerformerTag()
+{
+    for (Disc *disk : trackView->selectedDiscs()) {
+        disk->setAlbumPerformerTag(tagAlbumPerformerEdit->text());
+    }
+    trackView->updateAll();
+}
+
+/**************************************
+ *
+ **************************************/
+void MainWindow::setAlbumTag()
+{
+    for (Disc *disk : trackView->selectedDiscs()) {
+        disk->setAlbumTag(tagAlbumEdit->text());
+    }
+    trackView->updateAll();
+}
+
 /************************************************
 
  ************************************************/
@@ -1194,6 +1195,8 @@ void MainWindow::initActions()
     connect(actionAbout, &QAction::triggered, this, &MainWindow::openAboutDialog);
     actionAbout->setMenuRole(QAction::AboutRole);
 
+    connect(actionShowLogs, &QAction::triggered, this, &MainWindow::openLogViewDialog);
+
     Controls::arangeTollBarButtonsWidth(toolBar);
 
     actionWarnings->setIcon(Icon("warning"));
@@ -1262,7 +1265,7 @@ void MainWindow::loadSettings()
     this->setGeometry(x, y, width, height);
 
     splitter->restoreState(settings->value("MainWindow/Splitter").toByteArray());
-    trackView->header()->restoreState(settings->value("MainWindow/TrackView").toByteArray());
+    trackView->header()->restoreState(settings->value("MainWindow/TrackViewV2").toByteArray());
 
     outDirEdit->setHistory(Settings::i()->value(SETTINGS_OUTFILES_DIR_HISTORY_KEY).toStringList());
     outPatternEdit->setHistory(Settings::i()->value(SETTINGS_PATTERN_HISTORY_KEY).toStringList());
@@ -1280,7 +1283,7 @@ void MainWindow::saveSettings()
     settings->setValue("MainWindow/Width", QVariant(size().width()));
     settings->setValue("MainWindow/Height", QVariant(size().height()));
     settings->setValue("MainWindow/Splitter", QVariant(splitter->saveState()));
-    settings->setValue("MainWindow/TrackView", QVariant(trackView->header()->saveState()));
+    settings->setValue("MainWindow/TrackViewV2", QVariant(trackView->header()->saveState()));
 
     Settings::i()->setValue(SETTINGS_OUTFILES_DIR_HISTORY_KEY, outDirEdit->history());
     Settings::i()->setValue(SETTINGS_PATTERN_HISTORY_KEY, outPatternEdit->history());
@@ -1340,8 +1343,9 @@ static QStringList diskMsgsToHtml(int diskNum, const Disk *disk, const QStringLi
 {
     QStringList res;
     res << "<div>";
-    if (disk->count()) {
-        res << "<b>" + MainWindow::tr("Disk %1 \"%2 - %3\"", "Error message, %1, %2 and %3 is the number, artist and album for the disc, respectively").arg(diskNum).arg(disk->track(0)->artist(), disk->track(0)->album()) + "</b>";
+    if (disk->tracks().count()) {
+        QString artist = disk->isEmpty() ? "" : disk->tracks().first()->performerTag();
+        res << "<b>" + MainWindow::tr("Disk %1 \"%2 - %3\"", "Error message, %1, %2 and %3 is the number, artist and album for the disc, respectively").arg(diskNum).arg(artist, disk->albumTag()) + "</b>";
     }
     else {
         res << "<b>" + MainWindow::tr("Disk %1", "Error message, %1 is the disc number").arg(diskNum) + "</b>";
@@ -1350,7 +1354,7 @@ static QStringList diskMsgsToHtml(int diskNum, const Disk *disk, const QStringLi
     res << "<ul>";
     for (QString msg : msgs) {
         msg = msg.replace("\n", "<br>");
-        res << QString("<li>%1</li>").arg(msg);
+        res << QStringLiteral("<li>%1</li>").arg(msg);
     }
     res << "</ul>";
     res << "</div>";
