@@ -79,10 +79,12 @@ private slots:
 
     void removeDiscs();
 
+    void setGenreTag();
+    void setDateTag();
+    void setPerformerTag();
+    void setAlbumPerformerTag();
+    void setAlbumTag();
     void setStartTrackNum();
-    void setTrackTag();
-    void setDiscTag();
-    void setDiscTagInt();
 
     void configure();
     void configureEncoder();
@@ -91,6 +93,7 @@ private slots:
     void openScanDialog();
 
     void openAboutDialog();
+    void openLogViewDialog();
     void checkUpdates();
 
     void trackViewMenu(const QPoint &pos);
