@@ -29,6 +29,7 @@
 #include <QDialog>
 #include <QList>
 #include "tags.h"
+#include "controls.h"
 
 class Track;
 class Disc;
@@ -43,22 +44,19 @@ class TagEditor : public QDialog
     Q_OBJECT
 
 public:
-    explicit TagEditor(const QList<Track *> &tracks, const QList<Disc *> &discs, QWidget *parent = nullptr);
+    explicit TagEditor(const QList<Track *> &tracks, QWidget *parent = nullptr);
     ~TagEditor();
 
 public slots:
     void done(int res) override;
 
 private:
-    Ui::TagEditor *      ui;
-    void                 add2Widget(QWidget *widget1, QWidget *widget2, const QString &label);
-    void                 addLineEdit(TagId tagId, const QString &label);
-    void                 addTextEdit(TagId tagId, const QString &label);
-    void                 addIntEdit(TagId tagId, const QString &label);
-    void                 addIntEditNumCount(TagId numTagId, TagId cntTagId, const QString &numLabel);
+    Ui::TagEditor       *ui;
     const QList<Track *> mTracks;
-    const QList<Disc *>  mDiscs;
-    MultiValuesSpinBox * mStartTrackSpin;
+    const QList<Disk *>  mDisks;
+
+    void updateWidgets();
+    void apply();
 };
 
 #endif // TAGEDITOR_H
