@@ -26,7 +26,6 @@
 #include "trackviewdelegate.h"
 #include "trackview.h"
 #include "trackviewmodel.h"
-#include "project.h"
 #include "types.h"
 #include "icon.h"
 
@@ -38,6 +37,7 @@
 #include <QMouseEvent>
 #include <QApplication>
 #include <QToolTip>
+#include <QLineEdit>
 
 #include <QDebug>
 
@@ -84,6 +84,8 @@ public:
     TrackViewCacheItem *item(const QModelIndex &index);
     Keys                clickType(const QModelIndex &index, const QPoint &point);
 
+    QList<QModelIndex> keys() const { return mItems.keys(); }
+
 private:
     TrackViewCacheItem                       nullItemCache;
     QHash<QModelIndex, TrackViewCacheItem *> mItems;
@@ -124,7 +126,8 @@ TrackViewCacheItem *TrackViewCache::item(const QModelIndex &index)
 TrackViewDelegate::TrackViewDelegate(TrackView *parent) :
     QStyledItemDelegate(parent),
     mTrackView(parent),
-    mCache(new TrackViewCache)
+    mCache(new TrackViewCache),
+    mDownloadMovie(QSize(32, 32))
 {
     mTrackBtnPix   = Pixmap("cue-button", BUTTON_SIZE, BUTTON_SIZE);
     mAudioBtnPix   = Pixmap("audio-button", BUTTON_SIZE, BUTTON_SIZE);
@@ -134,8 +137,16 @@ TrackViewDelegate::TrackViewDelegate(TrackView *parent) :
     mTrackErrorPix = Pixmap("track-cancel", LINE_MARK_HEIGHT, LINE_MARK_HEIGHT);
     mNoCoverImg    = QImage(":noCover");
 
-    mDownloadMovie.setFileName(":wait");
-    connect(&mDownloadMovie, &QMovie::updated,
+    mDownloadMovie.loadFrame("wait-0");
+    mDownloadMovie.loadFrame("wait-1");
+    mDownloadMovie.loadFrame("wait-2");
+    mDownloadMovie.loadFrame("wait-3");
+    mDownloadMovie.loadFrame("wait-4");
+    mDownloadMovie.loadFrame("wait-5");
+    mDownloadMovie.loadFrame("wait-6");
+    mDownloadMovie.loadFrame("wait-7");
+
+    connect(&mDownloadMovie, &Movie::updated,
             this, &TrackViewDelegate::movieUpdated);
 }
 
@@ -180,6 +191,20 @@ void TrackViewDelegate::drawBranch(QPainter *painter, const QRect &rect, const Q
         drawSelectionMark(painter, rect);
 }
 
+/**************************************
+ *
+ **************************************/
+QWidget *TrackViewDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    QWidget   *res      = QStyledItemDelegate::createEditor(parent, option, index);
+    QLineEdit *lineEdit = qobject_cast<QLineEdit *>(res);
+
+    if (lineEdit) {
+        lineEdit->setPlaceholderText(index.data(TrackViewModel::RolePlaceHolder).toString());
+    }
+    return res;
+}
+
 /************************************************
 
  ************************************************/
@@ -222,7 +247,6 @@ void TrackViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
  ************************************************/
 void TrackViewDelegate::paintTrack(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-
     if (!(index.row() % 2))
         painter->fillRect(option.rect, QColor(128, 128, 128, 20));
 
@@ -296,7 +320,7 @@ void TrackViewDelegate::paintTrack(QPainter *painter, const QStyleOptionViewItem
         opt.minimum  = 0;
         opt.maximum  = 100;
         opt.progress = progress;
-        opt.text     = QString("%1 %2%").arg(txt).arg(opt.progress);
+        opt.text     = QStringLiteral("%1 %2%").arg(txt).arg(opt.progress);
 
         QApplication::style()->drawControl(QStyle::CE_ProgressBarContents, &opt, painter);
         QApplication::style()->drawControl(QStyle::CE_ProgressBarLabel, &opt, painter);
@@ -429,6 +453,8 @@ void TrackViewDelegate::paintDisc(QPainter *painter, const QStyleOptionViewItem
     cache->audioShowMenu = showAudioMenu;
     cache->isWaiting     = isWaiting;
     cache->markBtn       = markRect;
+
+    mDownloadMovie.setRunning(qobject_cast<TrackViewModel *>(mTrackView->model())->downloading());
 }
 
 /************************************************
@@ -467,7 +493,7 @@ QRect TrackViewDelegate::drawTitle(QPainter *painter, const QRect &windowRect, c
 
     QString text;
     if (!album.isEmpty() || !artist.isEmpty()) {
-        text = QString("%1 / %2").arg(artist, album);
+        text = QStringLiteral("%1 / %2").arg(artist, album);
     }
 
     QRect res;
@@ -484,7 +510,6 @@ QRect TrackViewDelegate::drawMark(QPainter *painter, bool isWaiting, const QRect
     QRect markRect(imgRect.right() - MARK_HEIGHT, imgRect.bottom() - MARK_HEIGHT, MARK_HEIGHT, MARK_HEIGHT);
 
     if (isWaiting) {
-        mDownloadMovie.start();
         painter->drawPixmap(markRect, mDownloadMovie.currentPixmap());
         return markRect;
     }
@@ -709,14 +734,14 @@ bool TrackViewDelegate::helpEvent(QHelpEvent *event, QAbstractItemView *view, co
             html += tr("<b>The conversion is not possible.</b>");
             html += "<ul>";
             for (const QString &s : errs) {
-                html += QString("<li><nobr>%1</nobr></li>").arg(s);
+                html += QStringLiteral("<li><nobr>%1</nobr></li>").arg(s);
             }
             html += "</ul>";
         }
         else {
             html += "<ul>";
             for (const QString &s : warns) {
-                html += QString("<li><nobr>%1</nobr></li>").arg(s);
+                html += QStringLiteral("<li><nobr>%1</nobr></li>").arg(s);
             }
             html += "</ul>";
         }
@@ -737,21 +762,11 @@ bool TrackViewDelegate::helpEvent(QHelpEvent *event, QAbstractItemView *view, co
  ************************************************/
 void TrackViewDelegate::movieUpdated()
 {
-    TrackViewModel *model = qobject_cast<TrackViewModel *>(mTrackView->model());
-    if (!model)
-        return;
-
-    bool active = false;
-    for (int i = 0; i < model->rowCount(QModelIndex()); ++i) {
-        QModelIndex index = model->index(i, 0, QModelIndex());
+    for (const QModelIndex &index : mCache->keys()) {
         if (mCache->item(index)->isWaiting) {
-            Project::instance()->emitDiscChanged(Project::instance()->disc(0));
-            active = true;
+            emit mTrackView->model()->dataChanged(index, index);
         }
     }
-
-    if (!active)
-        mDownloadMovie.stop();
 }
 
 /************************************************
