@@ -30,7 +30,7 @@
 #include <QImage>
 #include <QPixmap>
 #include <QRect>
-#include <QMovie>
+#include "movie.h"
 
 class TrackView;
 class TrackViewCache;
@@ -49,6 +49,8 @@ public:
 
     void drawBranch(QPainter *painter, const QRect &rect, const QModelIndex &index) const;
 
+    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
+
 signals:
     void trackButtonClicked(const QModelIndex &index, const QRect &buttonRect);
     void audioButtonClicked(const QModelIndex &index, int fileNum, const QRect &buttonRect);
@@ -73,7 +75,7 @@ private:
     QPixmap        mDiscWarnPix;
     QPixmap        mTrackOkPix;
     QPixmap        mTrackErrorPix;
-    mutable QMovie mDownloadMovie;
+    mutable Movie  mDownloadMovie;
     mutable int    mDiscHeightHint  = 0;
     mutable int    mAudioFileHeight = 0;
 
