@@ -51,8 +51,25 @@ class TrackViewModel::Cache
 {
 public:
     Cache() = default;
-    QSet<DiscNum>                downloadedDiscs;
-    QHash<Track, CacheTrackData> tracks;
+    QSet<DiscNum> downloadedDiscs;
+
+    CacheTrackData get(const Track &track)
+    {
+        return mTracks.value(QPair<Disc *, int>(track.disc(), track.index()));
+    }
+
+    void set(const Track &track, const CacheTrackData &data)
+    {
+        mTracks[QPair<Disc *, int>(track.disc(), track.index())] = data;
+    }
+
+    void remove(const Track &track)
+    {
+        mTracks.remove(QPair<Disc *, int>(track.disc(), track.index()));
+    }
+
+private:
+    QHash<QPair<Disk *, int>, CacheTrackData> mTracks;
 };
 
 class IndexData
@@ -89,8 +106,9 @@ public:
 
     Disc *disc() const
     {
-        if (mDiscId && mDiscId - 1 < Project::instance()->count())
+        if (mDiscId && mDiscId - 1 < Project::instance()->count()) {
             return Project::instance()->disc(mDiscId - 1);
+        }
 
         return nullptr;
     }
@@ -99,8 +117,9 @@ public:
     {
         if (mTrackId) {
             Disc *disc = this->disc();
-            if (disc && mTrackId - 1 < disc->count())
-                return disc->track(mTrackId - 1);
+            if (disc && mTrackId - 1 < disc->tracks().count()) {
+                return disc->tracks().at(mTrackId - 1);
+            }
         }
         return nullptr;
     }
@@ -110,14 +129,6 @@ private:
     quint16 mTrackId;
 };
 
-/************************************************
- *
- ************************************************/
-inline uint qHash(const Track &track, uint seed)
-{
-    return qHash(QPair<int, intptr_t>(track.trackNum(), intptr_t(track.disc())), seed);
-}
-
 /************************************************
 
  ************************************************/
@@ -129,11 +140,11 @@ TrackViewModel::TrackViewModel(TrackView *parent) :
     connect(Project::instance(), &Project::discChanged,
             this, &TrackViewModel::discDataChanged);
 
-    connect(Project::instance(), &Project::layoutChanged,
-            [this]() { this->layoutChanged(); });
+    connect(Project::instance(), &Project::layoutChanged, this,
+            [this]() { emit this->layoutChanged(); });
 
-    connect(Project::instance(), &Project::afterRemoveDisc,
-            [this]() { this->layoutChanged(); });
+    connect(Project::instance(), &Project::afterRemoveDisc, this,
+            [this]() { emit this->layoutChanged(); });
 
     connect(Project::instance(), &Project::beforeRemoveDisc,
             this, &TrackViewModel::invalidateCache);
@@ -155,22 +166,21 @@ QVariant TrackViewModel::headerData(int section, Qt::Orientation orientation, in
     if (orientation != Qt::Horizontal || role != Qt::DisplayRole)
         return QVariant();
 
+    // clang-format off
     switch (section) {
-        case TrackView::ColumnTracknum:
-            return QVariant(tr("Track", "Table header."));
-        case TrackView::ColumnDuration:
-            return QVariant(tr("Length", "Table header."));
-        case TrackView::ColumnTitle:
-            return QVariant(tr("Title", "Table header."));
-        case TrackView::ColumnArtist:
-            return QVariant(tr("Artist", "Table header."));
-        case TrackView::ColumnAlbum:
-            return QVariant(tr("Album", "Table header."));
-        case TrackView::ColumnComment:
-            return QVariant(tr("Comment", "Table header."));
-        case TrackView::ColumnFileName:
-            return QVariant(tr("File", "Table header."));
+        case TrackView::ColumnTracknum:   return tr("Track", "Table header.");
+        case TrackView::ColumnDuration:   return tr("Length", "Table header.");
+        case TrackView::ColumnTitle:      return tr("Title", "Table header.");
+        case TrackView::ColumnArtist:     return tr("Artist", "Table header.");
+        case TrackView::ColumnAlbum:      return tr("Album", "Table header.");
+        case TrackView::ColumnComment:    return tr("Comment", "Table header.");
+        case TrackView::ColumnFileName:   return tr("File", "Table header.");
+        case TrackView::ColumnDate:       return tr("Date", "Table header.");
+        case TrackView::ColumnGenre:      return tr("Genre", "Table header.");
+        case TrackView::ColumnSongWriter: return tr("Song writer", "Table header.");
     }
+    // clang-format on
+
     return QVariant();
 }
 
@@ -208,27 +218,33 @@ QModelIndex TrackViewModel::index(const Disc &disc, int col) const
  ************************************************/
 QModelIndex TrackViewModel::index(const Track &track, int col) const
 {
-    CacheTrackData &cache = mCache->tracks[track];
-
-    const IndexData indexData(cache.discNum, cache.trackNum);
-    if (indexData.track() && *indexData.track() == track) {
-        return index(cache.trackNum, col, index(cache.discNum, 0));
+    QModelIndex parentIndex = index(*(track.disc()));
+    if (parentIndex.isValid()) {
+        return index(track.index(), col, parentIndex);
     }
-
-    // Update cache
-    for (int d = 0; d < rowCount(); ++d) {
-        QModelIndex discIndex = index(d, 0);
-        for (int t = 0; t < rowCount(discIndex); ++t) {
-            IndexData indexData(d, t);
-            if (indexData.track() && *indexData.track() == track) {
-                cache.discNum  = d;
-                cache.trackNum = t;
-                return index(cache.trackNum, col, discIndex);
-            }
-        }
-    }
-
     return QModelIndex();
+
+    // CacheTrackData &cache = mCache->tracks[track];
+
+    // const IndexData indexData(cache.discNum, cache.trackNum);
+    // if (indexData.track() && *indexData.track() == track) {
+    //     return index(cache.trackNum, col, index(cache.discNum, 0));
+    // }
+
+    // // Update cache
+    // for (int d = 0; d < rowCount(); ++d) {
+    //     QModelIndex discIndex = index(d, 0);
+    //     for (int t = 0; t < rowCount(discIndex); ++t) {
+    //         IndexData indexData(d, t);
+    //         if (indexData.track() && *indexData.track()->index() == track.index()) {
+    //             cache.discNum  = d;
+    //             cache.trackNum = t;
+    //             return index(cache.trackNum, col, discIndex);
+    //         }
+    //     }
+    // }
+
+    // return QModelIndex();
 }
 
 /************************************************
@@ -280,23 +296,17 @@ bool TrackViewModel::setData(const QModelIndex &index, const QVariant &value, in
 
     QList<Track *> tracks = view()->selectedTracks();
     foreach (Track *track, tracks) {
+        // clang-format off
         switch (index.column()) {
-            case TrackView::ColumnTitle:
-                track->setTitle(value.toString());
-                break;
-
-            case TrackView::ColumnArtist:
-                track->setArtist(value.toString());
-                break;
-
-            case TrackView::ColumnAlbum:
-                track->setAlbum(value.toString());
-                break;
-
-            case TrackView::ColumnComment:
-                track->setComment(value.toString());
-                break;
+            case TrackView::ColumnTitle:        track->setTitleTag(value.toString());         break;
+            case TrackView::ColumnArtist:       track->setPerformerTag(value.toString());     break;
+            case TrackView::ColumnAlbum:        track->disk()->setAlbumTag(value.toString()); break;
+            case TrackView::ColumnComment:      track->setCommentTag(value.toString());       break;
+            case TrackView::ColumnDate:         track->setDateTag(value.toString());          break;
+            case TrackView::ColumnGenre:        track->setGenreTag(value.toString());         break;
+            case TrackView::ColumnSongWriter:   track->setSongWriterTag(value.toString());    break;
         }
+        // clang-format on
     }
 
     emit dataChanged(index, index, QVector<int>() << role);
@@ -308,35 +318,27 @@ bool TrackViewModel::setData(const QModelIndex &index, const QVariant &value, in
  ************************************************/
 QVariant TrackViewModel::trackData(const Track *track, const QModelIndex &index, int role) const
 {
-    // Display & Edit :::::::::::::::::::::::::::::::::::
-    if (role == Qt::DisplayRole || role == Qt::EditRole) {
+    // Text roles :::::::::::::::::::::::::::::::
+    if (role == Qt::DisplayRole || role == Qt::EditRole || role == TrackViewModel::RolePlaceHolder) {
+        // clang-format off
         switch (index.column()) {
-            case TrackView::ColumnTracknum:
-                return QVariant(QString("%1").arg(track->trackNum(), 2, 10, QChar('0')));
-
-            case TrackView::ColumnDuration:
-                return QVariant(trackDurationToString(track->duration()) + " ");
-
-            case TrackView::ColumnTitle:
-                return QVariant(track->title());
-
-            case TrackView::ColumnArtist:
-                return QVariant(track->artist());
-
-            case TrackView::ColumnAlbum:
-                return QVariant(track->album());
-
-            case TrackView::ColumnComment:
-                return QVariant(track->comment());
-
-            case TrackView::ColumnFileName:
-                return QVariant(Project::instance()->profile()->resultFileName(track));
+            case TrackView::ColumnTracknum:     return QVariant(QStringLiteral("%1").arg(track->trackNumTag(), 2, 10, QChar('0')));
+            case TrackView::ColumnDuration:     return QVariant(trackDurationToString(track->duration()) + " ");
+            case TrackView::ColumnTitle:        return QVariant(track->titleTag());
+            case TrackView::ColumnArtist:       return track->artistTag();
+            case TrackView::ColumnAlbum:        return track->disc()->albumTag();
+            case TrackView::ColumnComment:      return QVariant(track->commentTag());
+            case TrackView::ColumnFileName:     return QVariant(Project::instance()->profile()->resultFileName(track));
+            case TrackView::ColumnDate:         return track->dateTag();
+            case TrackView::ColumnGenre:        return track->genreTag();
+            case TrackView::ColumnSongWriter:   return track->songWriterTag();
         }
+        // clang-format on
 
         return QVariant();
     }
 
-    // ToolTip ::::::::::::::::::::::::::::::::::::::::::
+    // ToolTip ::::::::::::::::::::::::::::::::::
     if (role == Qt::ToolTipRole) {
         switch (index.column()) {
             case TrackView::ColumnFileName:
@@ -357,30 +359,21 @@ QVariant TrackViewModel::trackData(const Track *track, const QModelIndex &index,
         return QVariant();
     }
 
+    // clang-format off
     switch (role) {
-        case RoleItemType:
-            return TrackItem;
-        case RolePercent:
-            return mCache->tracks[*track].percent;
-        case RoleStatus:
-            return int(mCache->tracks[*track].state);
-        case RoleTracknum:
-            return track->trackNum();
-        case RoleDuration:
-            return track->duration();
-        case RoleTitle:
-            return track->title();
-        case RoleArtist:
-            return track->tag(TagId::AlbumArtist);
-        case RoleAlbum:
-            return track->album();
-        case RoleComment:
-            return track->album();
-        case RoleFileName:
-            return Project::instance()->profile()->resultFileName(track);
-        default:
-            return QVariant();
+        case RoleItemType:  return TrackItem;
+        case RolePercent:   return mCache->get(*track).percent;
+        case RoleStatus:    return int(mCache->get(*track).state);
+//        case RoleTracknum:  return track->trackNumTag();
+//        case RoleDuration:  return track->duration();
+//        case RoleTitle:     return track->titleTag();
+//        case RoleArtist:    return track->disc()->albumTag();
+//        case RoleAlbum:     return track->albumTag();
+//        case RoleComment:   return track->commentTag();
+//        case RoleFileName:  return Project::instance()->profile()->resultFileName(track);
+        default:            return QVariant();
     }
+    // clang-format on
 
     return QVariant();
 }
@@ -392,25 +385,24 @@ QVariant TrackViewModel::discData(const Disc *disc, const QModelIndex &index, in
 {
     // Display & Edit :::::::::::::::::::::::::::::::::::
     if (role == Qt::DisplayRole || role == Qt::EditRole) {
-        if (!disc->count())
+        if (!disc->tracks().count())
             return QVariant();
 
         QSet<QString> values;
 
         switch (index.column()) {
             case TrackView::ColumnTitle:
-                for (int i = 0; i < disc->count(); ++i)
-                    values << disc->track(i)->title();
+                for (int i = 0; i < disc->tracks().count(); ++i)
+                    values << disc->tracks().at(i)->titleTag();
                 break;
 
             case TrackView::ColumnArtist:
-                for (int i = 0; i < disc->count(); ++i)
-                    values << disc->track(i)->tag(TagId::AlbumArtist);
+                for (int i = 0; i < disc->tracks().count(); ++i)
+                    values << disc->albumTag();
                 break;
 
             case TrackView::ColumnAlbum:
-                for (int i = 0; i < disc->count(); ++i)
-                    values << disc->track(i)->album();
+                values << disc->albumTag();
                 break;
         }
 
@@ -427,12 +419,11 @@ QVariant TrackViewModel::discData(const Disc *disc, const QModelIndex &index, in
     // clang-format off
     switch (role) {
         case RoleItemType:      return DiscItem;
-        case RoleTagSetTitle:   return disc->tagSetTitle();
+        case RoleTagSetTitle:   return disc->currentTagSet().title;
         case RoleAudioFileName: return disc->audioFileNames();
         case RoleHasWarnings:   return Project::instance()->validator().diskHasWarnings(disc);
         case RoleHasErrors:     return Project::instance()->validator().diskHasErrors(disc);
         case RoleIsDownloads:   return mCache->downloadedDiscs.contains(index.row());
-        case RoleCoverFile:     return disc->coverImageFile();
         case RoleCoverImg:      return disc->coverImagePreview();
         case RoleCueFilePath:   return disc->cueFilePath();
         case RoleAudioFilePath: return disc->audioFilePaths();
@@ -488,7 +479,7 @@ int TrackViewModel::rowCount(const QModelIndex &parent) const
 
     Disc *disc = data.disc();
     if (disc)
-        return disc->count();
+        return disc->tracks().count();
 
     return 0;
 }
@@ -509,6 +500,9 @@ Qt::ItemFlags TrackViewModel::flags(const QModelIndex &index) const
             case TrackView::ColumnArtist:
             case TrackView::ColumnAlbum:
             case TrackView::ColumnComment:
+            case TrackView::ColumnDate:
+            case TrackView::ColumnGenre:
+            case TrackView::ColumnSongWriter:
                 res = res | Qt::ItemIsEditable;
                 break;
         }
@@ -533,6 +527,14 @@ Track *TrackViewModel::trackByIndex(const QModelIndex &index)
     return IndexData(index).track();
 }
 
+/**************************************
+ *
+ **************************************/
+bool TrackViewModel::downloading() const
+{
+    return !mCache->downloadedDiscs.isEmpty();
+}
+
 /************************************************
  *
  ************************************************/
@@ -556,9 +558,10 @@ void TrackViewModel::downloadFinished(const Disc &disc)
  ************************************************/
 void TrackViewModel::trackProgressChanged(const Track &track, TrackState state, Percent percent)
 {
-    CacheTrackData &cache = mCache->tracks[track];
-    cache.state           = state;
-    cache.percent         = percent;
+    CacheTrackData cache = mCache->get(track);
+    cache.state          = state;
+    cache.percent        = percent;
+    mCache->set(track, cache);
 
     QModelIndex idx = index(track, TrackView::ColumnPercent);
     emit        dataChanged(idx, idx);
@@ -581,6 +584,7 @@ void TrackViewModel::invalidateCache(const Disc *disc)
 {
     mCache->downloadedDiscs.remove(index(*disc).row());
 
-    for (int i = 0; i < disc->count(); ++i)
-        mCache->tracks.remove(*(disc->track(i)));
+    for (const Track *track : disc->tracks()) {
+        mCache->remove(*(track));
+    }
 }
