@@ -63,7 +63,7 @@ public:
     Disc  *discByIndex(const QModelIndex &index);
     Track *trackByIndex(const QModelIndex &index);
 
-public slots:
+    bool downloading() const;
 
 protected:
     enum ItemType {
@@ -75,27 +75,21 @@ protected:
         RoleItemType = Qt::UserRole + 1,
         RolePercent,
         RoleStatus,
-        RoleTracknum,
-        RoleDuration,
-        RoleTitle,
-        RoleArtist,
-        RoleAlbum,
-        RoleComment,
-        RoleFileName,
-        RoleAudioFileName,
         RoleTagSetTitle,
-        RoleHasWarnings,
-        RoleHasErrors,
+
         RoleIsDownloads,
-        RoleItemID,
-        RoleTrack,
-        RoleCoverFile,
+
         RoleCoverImg,
         RoleCueFilePath,
+        RoleAudioFileName,
         RoleAudioFilePath,
+
+        RoleHasWarnings,
+        RoleHasErrors,
         RoleDiscWarnings,
         RoleDiscErrors,
-        RoleDiscPerformer
+
+        RolePlaceHolder,
     };
 
 public slots:
