@@ -31,10 +31,11 @@
 #include <QStringList>
 #include <QDebug>
 #include <QLoggingCategory>
-#include <functional>
+#include "profiles.h"
 
 #include "musicbrainz.h"
 #include "discogs.h"
+#include <QNetworkProxy>
 
 namespace {
 Q_LOGGING_CATEGORY(LOG, "DataProvider")
@@ -53,12 +54,33 @@ bool DataProvider::canDownload(const Disc &disk)
     return res;
 }
 
+static QNetworkProxy::ProxyType proxyTypeToQProxyType(ProxyType type)
+{
+    // clang-format off
+    switch (type) {
+        case ProxyType::NoProxy:     return QNetworkProxy::ProxyType::NoProxy;
+        case ProxyType::HttpProxy:   return QNetworkProxy::ProxyType::HttpProxy;
+        case ProxyType::Socks5Proxy: return QNetworkProxy::ProxyType::Socks5Proxy;
+    }
+    // clang-format on
+
+    return QNetworkProxy::ProxyType::NoProxy;
+}
+
 /************************************************
 
  ************************************************/
-DataProvider::DataProvider(QObject *parent) :
+DataProvider::DataProvider(Profile *profile, QObject *parent) :
     QObject(parent)
 {
+
+    QNetworkProxy proxy;
+    proxy.setType(proxyTypeToQProxyType(profile->proxyType()));
+    proxy.setHostName(profile->proxyHost());
+    proxy.setPort(profile->proxyPort());
+    proxy.setUser(profile->proxyUserName());
+    proxy.setPassword(profile->proxyPassword());
+    QNetworkProxy::setApplicationProxy(proxy);
 }
 
 /************************************************
@@ -111,7 +133,7 @@ bool DataProvider::isFinished() const
 /************************************************
 
  ************************************************/
-void DataProvider::serviceFinished(const QVector<Tracks> &result)
+void DataProvider::serviceFinished(const QVector<InternetTags> &result)
 {
     mResult << result;
 
@@ -155,7 +177,10 @@ void InterntService::stop()
 QNetworkReply *InterntService::get(const QNetworkRequest &request)
 {
     QNetworkRequest req = request;
-    req.setRawHeader("User-Agent", QString("Flacon/%1 (https://github.com/flacon/flacon)").arg(FLACON_VERSION).toUtf8());
+    req.setRawHeader("User-Agent", QStringLiteral("Flacon/%1 (https://github.com/flacon/flacon)").arg(FLACON_VERSION).toUtf8());
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    req.setTransferTimeout(3 * 1000);
+#endif
     qCDebug(LOG).noquote() << req.url().toEncoded();
 
     QNetworkReply *reply = networkAccessManager()->get(req);
@@ -174,6 +199,7 @@ void InterntService::error(const QString &message)
         }
     }
 
+    qCDebug(LOG) << "(" << objectName() << ") Request error" << message;
     emit errorOccurred(message);
 }
 
@@ -193,10 +219,44 @@ void InterntService::removeDuplicates()
 {
     for (int i = mResult.size() - 1; i >= 0; --i) {
         for (int j = 0; j < i; ++j) {
-            if (mResult[i] == mResult[j]) {
+            if (compareTags(mResult[i], mResult[j])) {
                 mResult.removeAt(i);
                 break;
             }
         }
     }
 }
+
+/**************************************
+
+ **************************************/
+bool InterntService::compareTags(const InternetTags &tags1, const InternetTags &tags2) const
+{
+    bool res = true;
+
+    res = res && tags1.album() == tags2.album();
+    res = res && tags1.albumPerformer() == tags2.albumPerformer();
+
+    res = res && tags1.tracks().count() == tags2.tracks().count();
+
+    int n = -1;
+    for (const Tags::Track &t : tags1.tracks()) {
+        n++;
+        res = res && compareTrackTags(t, tags2.tracks().at(n));
+    }
+
+    return res;
+}
+
+/**************************************
+
+ **************************************/
+bool InterntService::compareTrackTags(const Tags::Track &tags1, const Tags::Track &tags2) const
+{
+    bool res = true;
+
+    res = res && tags1.title() == tags2.title();
+    res = res && tags1.trackNum() == tags2.trackNum();
+
+    return res;
+}
