@@ -30,11 +30,12 @@
 #include <QList>
 #include <QtNetwork/QNetworkRequest>
 #include <QtNetwork/QNetworkReply>
-#include "track.h"
+#include "tags.h"
 
 class Disc;
 class QNetworkAccessManager;
 class InterntService;
+class Profile;
 
 class DataProvider : public QObject
 {
@@ -43,7 +44,7 @@ public:
     static bool canDownload(const Disc &disk);
 
 public:
-    explicit DataProvider(QObject *parent = nullptr);
+    explicit DataProvider(Profile *profile, QObject *parent = nullptr);
     virtual ~DataProvider();
 
     void start(const Disc &disk);
@@ -52,14 +53,14 @@ public:
     bool isFinished() const;
 
 signals:
-    void finished(const QVector<Tracks> &result);
+    void finished(const QVector<InternetTags> &result);
     void errorOccurred(const QString &err);
 
 private:
     QList<InterntService *> mServices;
-    QVector<Tracks>         mResult;
+    QVector<InternetTags>   mResult;
 
-    void serviceFinished(const QVector<Tracks> &result);
+    void serviceFinished(const QVector<InternetTags> &result);
 };
 
 class InterntService : public QObject
@@ -74,12 +75,12 @@ public:
     virtual void stop();
 
 signals:
-    void finished(const QVector<Tracks> result);
+    void finished(const QVector<InternetTags> result);
     void errorOccurred(const QString &err);
 
 protected:
     const Disc            &mDisk;
-    QVector<Tracks>        mResult;
+    QVector<InternetTags>  mResult;
     QList<QNetworkReply *> mReplies;
 
     QNetworkAccessManager *networkAccessManager() const;
@@ -87,6 +88,8 @@ protected:
 
     void error(const QString &message);
     void removeDuplicates();
+    bool compareTags(const InternetTags &tags1, const InternetTags &tags2) const;
+    bool compareTrackTags(const InternetTags::Track &tags1, const InternetTags::Track &tags2) const;
 };
 
 #endif // DATAPROVIDER_H
