@@ -33,6 +33,7 @@
 #include "scanner.h"
 #include "consoleout.h"
 #include "types.h"
+#include "debug.h"
 
 #include <QString>
 #include <QLocale>
@@ -110,17 +111,6 @@ void printVersion()
     out << "There is NO WARRANTY, to the extent permitted by law." << Qt::endl;
 }
 
-/************************************************
- *
- ************************************************/
-void consoleErroHandler(QtMsgType type, const QMessageLogContext &context, const QString &message)
-{
-    Q_UNUSED(type)
-    Q_UNUSED(context)
-
-    QTextStream(stderr) << htmlToText(message).toLocal8Bit() << Qt::endl;
-}
-
 namespace LibraryInfo {
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QString path(QLibraryInfo::LibraryLocation p)
@@ -156,7 +146,7 @@ void translate(QApplication *app)
     }
 
     QTranslator *appTranslator = new QTranslator(app);
-    if (appTranslator->load(QString("flacon_%2.qm").arg(locale)) || appTranslator->load(QString("%1/flacon_%2.qm").arg(appDir, locale))) {
+    if (appTranslator->load(QStringLiteral("flacon_%2.qm").arg(locale)) || appTranslator->load(QStringLiteral("%1/flacon_%2.qm").arg(appDir, locale))) {
         app->installTranslator(appTranslator);
     }
 }
@@ -166,7 +156,6 @@ void translate(QApplication *app)
  ************************************************/
 int runConsole(int argc, char *argv[], const QStringList &files)
 {
-    qInstallMessageHandler(consoleErroHandler);
     QCoreApplication app(argc, argv);
 
     Project::instance()->load(Settings::i());
@@ -221,17 +210,13 @@ int runConsole(int argc, char *argv[], const QStringList &files)
         }
     }
 
-    app.connect(&converter, &Conv::Converter::finished,
-                &app, &QCoreApplication::quit);
+    app.connect(&converter, &Conv::Converter::finished, [](bool success) {
+        qApp->exit(success ? 0 : 11);
+    });
 
-    app.connect(&converter, &Conv::Converter::error,
-                [](const QString &message) {
-                    consoleErroHandler(QtCriticalMsg, QMessageLogContext(), message);
-                });
-
-    converter.start(*(Project::instance()->profile()));
-    if (!converter.isRunning())
-        return 11;
+    QTimer::singleShot(0, &converter, [&converter]() {
+        converter.start(*(Project::instance()->profile()));
+    });
 
     return app.exec();
 }
@@ -322,14 +307,7 @@ int main(int argc, char *argv[])
         Settings::setFileName(parser.value("config"));
     }
 
-    if (parser.isSet("debug") || getenv("FLACON_DEBUG")) {
-        setenv("QT_LOGGING_RULES", "*.debug=true;qt.*.debug=false;kf.*.debug=false;", 1);
-        qSetMessagePattern("%{time yyyy.MM.dd hh:mm:ss.zzz t} [%{threadid}] %{type}: %{category}: %{message}");
-    }
-    else {
-        setenv("QT_LOGGING_RULES", "*.debug=false;default.debug=true;", 1);
-        qSetMessagePattern("%{if-warning}Warning: %{endif}%{if-critical}Error: %{endif}%{if-fatal}Error: %{endif}%{message}");
-    }
+    initDebug((parser.isSet("debug") || getenv("FLACON_DEBUG")));
 
     quiet    = parser.isSet("quiet");
     progress = parser.isSet("progress");
