@@ -41,56 +41,37 @@ class Track;
  ************************************************/
 class PatternExpander
 {
+public:
+    enum class Mode {
+        Album,
+        Track,
+    };
+
 public:
     PatternExpander();
+    PatternExpander(const AlbumTags &albumTags, const TrackTags &trackTags, const TrackTags &firstTrackTags);
     PatternExpander(const Track &track);
 
-    TrackCount trackCount() const { return mTrackCount; }
-    void       setTrackCount(TrackCount value) { mTrackCount = value; }
-
-    TrackNum trackNum() const { return mTrackNum; }
-    void     setTrackNum(TrackNum value) { mTrackNum = value; }
-
-    DiscCount discCount() const { return mDiscCount; }
-    void      setDiscCount(DiscCount value) { mDiscCount = value; }
-
-    DiscNum discNum() const { return mDiscNum; }
-    void    setDiscNum(DiscNum value) { mDiscNum = value; }
-
-    QString artist() const { return mArtist; }
-    void    setArtist(const QString &value) { mArtist = value; }
-
-    QString album() const { return mAlbum; }
-    void    setAlbum(const QString &value) { mAlbum = value; }
-
-    QString trackTitle() const { return mTrackTitle; }
-    void    setTrackTtle(const QString &value) { mTrackTitle = value; }
-
-    QString genre() const { return mGenre; }
-    void    setGenre(const QString &value) { mGenre = value; }
-
-    QString date() const { return mDate; }
-    void    setDate(const QString &value) { mDate = value; }
-
     QString expand(const QString &pattern) const;
 
     static QString resultFileName(const QString &pattern, const Track *track, const QString &ext);
 
     static QString example(const QString &pattern);
 
+    const AlbumTags &albumTags() const { return mAlbumTags; }
+    AlbumTags       &albumTags() { return mAlbumTags; }
+
+    const TrackTags trackTags() const { return mTrackTags; }
+    TrackTags       trackTags() { return mTrackTags; }
+
 protected:
     static int lastDirSeparattor(const QString &pattern);
+    QString    expand(const QString &pattern, Mode mode) const;
 
 private:
-    TrackCount mTrackCount;
-    TrackNum   mTrackNum;
-    DiscCount  mDiscCount;
-    DiscNum    mDiscNum;
-    QString    mAlbum;
-    QString    mTrackTitle;
-    QString    mArtist;
-    QString    mGenre;
-    QString    mDate;
+    AlbumTags mAlbumTags;
+    TrackTags mTrackTags;
+    TrackTags mFirstTrackTags;
 };
 
 #endif // PATTERNEXPANDER_H
