@@ -27,34 +27,71 @@
 #include "extprogram.h"
 
 static constexpr auto PROFILES_GROUP = "Profiles";
+#if !BUNDLED_PROGRAMS
 static constexpr auto PROGRAMS_GROUP = "Programs";
+#endif
+static constexpr auto CURRENT_PROFILE_ID             = "OutFiles/Profile";
+static constexpr auto KNOWN_FORMATS_KEY              = "KnownFormats";
+static constexpr auto PROFILE_NAME_KEY               = "Name";
+static constexpr auto PROFILE_FORMAT_KEY             = "Format";
+static constexpr auto PROFILE_OUT_DIRECTORY_KEY      = "OutDirectory";
+static constexpr auto PROFILE_OUT_PATTERN_KEY        = "OutPattern";
+static constexpr auto PROFILE_BITS_PER_SAMPLE_KEY    = "BitsPerSample";
+static constexpr auto PROFILE_SAMPLE_RATE_KEY        = "SampleRate";
+static constexpr auto PROFILE_CREATE_CUE_KEY         = "CreateCue";
+static constexpr auto PROFILE_EMBED_CUE_KEY          = "EmbedCue";
+static constexpr auto PROFILE_CUE_FILE_NAME_KEY      = "CueFileName";
+static constexpr auto PROFILE_PREGAP_TYPE_KEY        = "PregapType";
+static constexpr auto PROFILE_REPLAY_GAIN_KEY        = "ReplayGain";
+static constexpr auto PROFILE_COVER_FILE_MODE_KEY    = "CoverFile/Mode";
+static constexpr auto PROFILE_COVER_FILE_SIZE_KEY    = "CoverFile/Size";
+static constexpr auto PROFILE_COVER_EMBED_MODE_KEY   = "CoverEmbed/Mode";
+static constexpr auto PROFILE_COVER_EMBED_SIZE_KEY   = "CoverEmbed/Size";
+static constexpr auto PROFILE_WRITE_DISK_NUM_TAG_KEY = "Tags/WriteSingleDiskNum";
 
-static constexpr auto CURRENT_PROFILE_ID           = "OutFiles/Profile";
-static constexpr auto KNOWN_FORMATS_KEY            = "KnownFormats";
-static constexpr auto PROFILE_NAME_KEY             = "Name";
-static constexpr auto PROFILE_FORMAT_KEY           = "Format";
-static constexpr auto PROFILE_OUT_DIRECTORY_KEY    = "OutDirectory";
-static constexpr auto PROFILE_OUT_PATTERN_KEY      = "OutPattern";
-static constexpr auto PROFILE_BITS_PER_SAMPLE_KEY  = "BitsPerSample";
-static constexpr auto PROFILE_SAMPLE_RATE_KEY      = "SampleRate";
-static constexpr auto PROFILE_CREATE_CUE_KEY       = "CreateCue";
-static constexpr auto PROFILE_EMBED_CUE_KEY        = "EmbedCue";
-static constexpr auto PROFILE_CUE_FILE_NAME_KEY    = "CueFileName";
-static constexpr auto PROFILE_PREGAP_TYPE_KEY      = "PregapType";
-static constexpr auto PROFILE_REPLAY_GAIN_KEY      = "ReplayGain";
-static constexpr auto PROFILE_COVER_FILE_MODE_KEY  = "CoverFile/Mode";
-static constexpr auto PROFILE_COVER_FILE_SIZE_KEY  = "CoverFile/Size";
-static constexpr auto PROFILE_COVER_EMBED_MODE_KEY = "CoverEmbed/Mode";
-static constexpr auto PROFILE_COVER_EMBED_SIZE_KEY = "CoverEmbed/Size";
-
-static constexpr auto DEFAULTCODEPAGE_KEY     = "Tags/DefaultCodepage";
 static constexpr auto SPLIT_TRACK_TITLE_KEY   = "Tags/SplitTrackTitle";
 static constexpr auto ENCODER_THREADCOUNT_KEY = "Encoder/ThreadCount";
 static constexpr auto ENCODER_TMPDIR_KEY      = "Encoder/TmpDir";
 
+static constexpr auto PROXY_TYPE = "Proxy/Type";
+static constexpr auto PROXY_HOST = "Proxy/Host";
+static constexpr auto PROXY_PORT = "Proxy/Port";
+static constexpr auto PROXY_USER = "Proxy/User";
+static constexpr auto PROXY_PASS = "Proxy/Password";
+
 QString   Settings::mFileName;
 Settings *Settings::mInstance = nullptr;
 
+/************************************************
+ *
+ ************************************************/
+static QString proxyTypeToStr(ProxyType type)
+{
+    // clang-format off
+    switch (type) {
+        case ProxyType::NoProxy:          return "NoProxy";
+        case ProxyType::HttpProxy:        return "HTTP";
+        case ProxyType::Socks5Proxy:      return "Socks5";
+    }
+    // clang-format on
+
+    return "Disabled";
+}
+
+/************************************************
+ *
+ ************************************************/
+ProxyType strToProxyType(const QString &value)
+{
+    QString s = value.toLower();
+    // clang-format off
+    if (s == "socks5") return ProxyType::Socks5Proxy;
+    if (s == "http")   return ProxyType::HttpProxy;
+    // clang-format on
+
+    return ProxyType::NoProxy;
+}
+
 /************************************************
  *
  ************************************************/
@@ -108,7 +145,7 @@ Settings::Settings(const QString &fileName) :
  ************************************************/
 Profile Settings::readProfile(const QString &profileId)
 {
-    QString group = QString("%1/%2").arg(PROFILES_GROUP, profileId);
+    QString group = QStringLiteral("%1/%2").arg(PROFILES_GROUP, profileId);
 
     beginGroup(group);
 
@@ -132,6 +169,7 @@ Profile Settings::readProfile(const QString &profileId)
 
     profile.setCreateCue(value(PROFILE_CREATE_CUE_KEY, profile.isCreateCue()).toBool());
     profile.setEmbedCue(value(PROFILE_EMBED_CUE_KEY, profile.isEmbedCue()).toBool());
+    profile.setWriteSingleDiskNum(value(PROFILE_WRITE_DISK_NUM_TAG_KEY, profile.isWriteSingleDiskNum()).toBool());
 
     CoverOptions opts = profile.copyCoverOptions();
     opts.mode         = strToCoverMode(value(PROFILE_COVER_FILE_MODE_KEY).toString(), opts.mode);
@@ -157,6 +195,12 @@ Profile Settings::readProfile(const QString &profileId)
     profile.setEncoderThreadsCount(readThreadsCount(ENCODER_THREADCOUNT_KEY, profile.encoderThreadsCount()));
     profile.setSplitTrackTitle(value(SPLIT_TRACK_TITLE_KEY, profile.isSplitTrackTitle()).toBool());
 
+    profile.setProxyType(strToProxyType(value(PROXY_TYPE).toString()));
+    profile.setProxyHost(value(PROXY_HOST).toString());
+    profile.setProxyPort(value(PROXY_PORT, 0).toInt());
+    profile.setProxyUserName(value(PROXY_USER).toString());
+    profile.setProxyPassword(value(PROXY_PASS).toString());
+
     return profile;
 }
 
@@ -169,7 +213,7 @@ void Settings::writeProfile(const Profile &profile)
         return;
     }
 
-    QString group = QString("%1/%2").arg(PROFILES_GROUP, profile.id());
+    QString group = QStringLiteral("%1/%2").arg(PROFILES_GROUP, profile.id());
     beginGroup(group);
 
     setValue(PROFILE_NAME_KEY, profile.name());
@@ -187,6 +231,7 @@ void Settings::writeProfile(const Profile &profile)
 
     setValue(PROFILE_CREATE_CUE_KEY, profile.isCreateCue());
     setValue(PROFILE_EMBED_CUE_KEY, profile.isEmbedCue());
+    setValue(PROFILE_WRITE_DISK_NUM_TAG_KEY, profile.isWriteSingleDiskNum());
 
     setValue(PROFILE_COVER_FILE_MODE_KEY, coverModeToString(profile.copyCoverOptions().mode));
     setValue(PROFILE_COVER_FILE_SIZE_KEY, profile.copyCoverOptions().size);
@@ -205,6 +250,12 @@ void Settings::writeProfile(const Profile &profile)
     setValue(ENCODER_TMPDIR_KEY, profile.tmpDir());
     setValue(ENCODER_THREADCOUNT_KEY, profile.encoderThreadsCount());
     setValue(SPLIT_TRACK_TITLE_KEY, profile.isSplitTrackTitle());
+
+    setValue(PROXY_TYPE, proxyTypeToStr(profile.proxyType()));
+    setValue(PROXY_HOST, profile.proxyHost());
+    setValue(PROXY_PORT, profile.proxyPort());
+    setValue(PROXY_USER, profile.proxyUserName());
+    setValue(PROXY_PASS, profile.proxyPassword());
 }
 
 /************************************************
@@ -341,7 +392,7 @@ void Settings::writeCurrentProfileId(const QString &profileId)
 void Settings::readExtPrograms() const
 {
     for (ExtProgram *p : ExtProgram::allPrograms()) {
-        auto key = QString("%1/%2").arg(PROGRAMS_GROUP, p->name());
+        auto key = QStringLiteral("%1/%2").arg(PROGRAMS_GROUP, p->name());
 
         QString path = value(key).toString();
         if (path.isEmpty()) {
@@ -358,24 +409,8 @@ void Settings::writeExtPrograms()
 {
     remove(PROGRAMS_GROUP);
     for (ExtProgram *p : ExtProgram::allPrograms()) {
-        auto key = QString("%1/%2").arg(PROGRAMS_GROUP, p->name());
+        auto key = QStringLiteral("%1/%2").arg(PROGRAMS_GROUP, p->name());
         setValue(key, p->path());
     }
 }
 #endif
-
-/************************************************
- *
- ************************************************/
-QString Settings::defaultCodepage() const
-{
-    return value(DEFAULTCODEPAGE_KEY, "AUTODETECT").toString();
-}
-
-/************************************************
- *
- ************************************************/
-void Settings::setDefaultCodepage(const QString &value)
-{
-    setValue(DEFAULTCODEPAGE_KEY, value);
-}
