@@ -29,6 +29,7 @@
 #include <QDir>
 #include <QDebug>
 #include <QRegularExpression>
+#include <QThread>
 
 /************************************************
  *
@@ -167,7 +168,7 @@ QString CueTime::toString(bool cdQuality) const
         int sec = (mCdValue - min * 60 * 75) / 75;
         int frm = mCdValue - (min * 60 + sec) * 75;
 
-        return QString("%1:%2:%3")
+        return QStringLiteral("%1:%2:%3")
                 .arg(min, 2, 10, QChar('0'))
                 .arg(sec, 2, 10, QChar('0'))
                 .arg(frm, 2, 10, QChar('0'));
@@ -177,7 +178,7 @@ QString CueTime::toString(bool cdQuality) const
         int sec  = (mHiValue - min * 60 * 1000) / 1000;
         int msec = mHiValue - (min * 60 + sec) * 1000;
 
-        return QString("%1:%2.%3")
+        return QStringLiteral("%1:%2.%3")
                 .arg(min, 2, 10, QChar('0'))
                 .arg(sec, 2, 10, QChar('0'))
                 .arg(msec, 3, 10, QChar('0'));
@@ -575,3 +576,64 @@ QString expandFilePath(const QString &path)
 
     return QFileInfo(res).absoluteFilePath();
 }
+
+QString firstNotNullString(const QString &s1, const QString &s2)
+{
+    // clang-format off
+    if (!s1.isNull()) return s1;
+    return s2;
+    // clang-format on
+}
+
+QString firstNotNullString(const QString &s1, const QString &s2, const QString &s3)
+{
+    // clang-format off
+    if (!s1.isNull()) return s1;
+    if (!s2.isNull()) return s2;
+    return s3;
+    // clang-format on
+}
+
+QString firstNotEmptyString(const QString &s1, const QString &s2)
+{
+    // clang-format off
+    if (!s1.isEmpty()) return s1;
+    return s2;
+    // clang-format on
+}
+
+QString firstNotEmptyString(const QString &s1, const QString &s2, const QString &s3)
+{
+    // clang-format off
+    if (!s1.isEmpty()) return s1;
+    if (!s2.isEmpty()) return s2;
+    return s3;
+    // clang-format on
+}
+
+QString trackStateToString(TrackState state)
+{
+    // clang-format off
+    switch (state) {
+        case TrackState::NotRunning: return "NotRunning";
+        case TrackState::Canceled:   return "Canceled";
+        case TrackState::Error:      return "Error";
+        case TrackState::Aborted:    return "Aborted";
+        case TrackState::OK:         return "OK";
+        case TrackState::Splitting:  return "Splitting";
+        case TrackState::Encoding:   return "Encoding";
+        case TrackState::Queued:     return "Queued";
+        case TrackState::WaitGain:   return "WaitGain";
+        case TrackState::CalcGain:   return "CalcGain";
+        case TrackState::WriteGain:  return "WriteGain";
+    };
+    // clang-format on
+    return "Unknown";
+}
+
+void Abort::check()
+{
+    if (QThread::currentThread()->isInterruptionRequested()) {
+        throw Abort();
+    }
+}
