@@ -40,28 +40,6 @@ using Duration   = uint;
 using mSec       = int;
 using Percent    = quint8;
 
-enum class TagId {
-    Album,
-    Catalog,
-    CDTextfile,
-    Comment,
-    Date,
-    Flags,
-    Genre,
-    ISRC,
-    Artist,
-    SongWriter,
-    Title,
-    DiscId,
-    File,
-    DiscNum,
-    DiscCount,
-    CueFile,
-    AlbumArtist,
-    TrackNum,
-    TrackCount,
-};
-
 enum class PreGapType {
     Skip = 0,
     ExtractToFile,
@@ -108,6 +86,14 @@ public:
         std::runtime_error(msg.toStdString()) { }
 };
 
+class Abort : public std::runtime_error
+{
+public:
+    Abort() :
+        std::runtime_error("") { }
+    static void check() noexcept(false);
+};
+
 class CueTime
 {
 public:
@@ -181,6 +167,8 @@ enum class TrackState {
 
 Q_DECLARE_METATYPE(TrackState)
 
+QString trackStateToString(TrackState state);
+
 enum class DiskState {
     NotRunning = 0,
     Canceled   = 1,
@@ -233,6 +221,12 @@ int calcSampleRate(int input, SampleRate resample);
 
 int calcQuality(int input, int preferences, int formatMax);
 
+enum class ProxyType {
+    NoProxy,
+    HttpProxy,
+    Socks5Proxy,
+};
+
 QByteArray leftPart(const QByteArray &line, const char separator);
 QByteArray rightPart(const QByteArray &line, const char separator);
 void       initTypes();
@@ -261,4 +255,10 @@ private:
 
 QString expandFilePath(const QString &path);
 
+QString firstNotNullString(const QString &s1, const QString &s2);
+QString firstNotNullString(const QString &s1, const QString &s2, const QString &s3);
+
+QString firstNotEmptyString(const QString &s1, const QString &s2);
+QString firstNotEmptyString(const QString &s1, const QString &s2, const QString &s3);
+
 #endif // TYPES_H
